'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fileloader = require('@wessberg/fileloader');
var filesaver = require('@wessberg/filesaver');
var chalk = _interopDefault(require('chalk'));
var zlib = require('zlib');
var iltorb = require('iltorb');
var core = require('@babel/core');
var astring = require('astring');
var rollup = require('rollup');
var commonjs = _interopDefault(require('rollup-plugin-commonjs'));
var nodeResolve = _interopDefault(require('rollup-plugin-node-resolve'));
var multiEntry = _interopDefault(require('rollup-plugin-multi-entry'));
var crypto = require('crypto');
var tempDir = _interopDefault(require('temp-dir'));
var path = require('path');
var browserslistGenerator = require('@wessberg/browserslist-generator');
var stringutil = require('@wessberg/stringutil');
var toposort = _interopDefault(require('toposort'));
var semver = require('semver');
var buffer = require('buffer');
var builder = _interopDefault(require('core-js-builder'));
var findUp = require('find-up');
var devcert = require('devcert');
var http2 = require('http2');
var http = require('http');
var https = require('https');
var url = require('url');
var httpStatusCodes = require('http-status-codes');
var tslib_1 = require('tslib');
var di = require('@wessberg/di');
var fs = require('fs');



var ContentEncodingKind;
(function (ContentEncodingKind) {
    ContentEncodingKind["GZIP"] = "gzip";
    ContentEncodingKind["BROTLI"] = "br";
})(ContentEncodingKind || (ContentEncodingKind = {}));





const stringTuple = (...args) => args;

const POLYFILL_CONTEXTS = stringTuple("node", "window", "worker");
const NODE_CONTEXT = new Set(stringTuple("node"));
const WINDOW_CONTEXT = new Set(stringTuple("window"));
const WORKER_CONTEXT = new Set(stringTuple("worker"));
const WINDOW_NODE_CONTEXT = new Set([...WINDOW_CONTEXT, ...NODE_CONTEXT]);
const WINDOW_WORKER_CONTEXT = new Set([...WINDOW_CONTEXT, ...WORKER_CONTEXT]);
const ALL_CONTEXTS = new Set([...NODE_CONTEXT, ...WINDOW_WORKER_CONTEXT]);



















const environmentDefaults = {
    NPM_PACKAGE_NAME: "",
    NPM_PACKAGE_VERSION: "",
    NPM_PACKAGE_HOMEPAGE: "",
    NPM_PACKAGE_MAIN: "",
    NPM_PACKAGE_DEPENDENCIES_INTERSECTION_OBSERVER: "",
    NPM_PACKAGE_DEPENDENCIES_CONSTRUCT_STYLE_SHEETS_POLYFILL: "",
    NPM_PACKAGE_DEPENDENCIES_SCROLL_BEHAVIOR_POLYFILL: "",
    NPM_PACKAGE_DEPENDENCIES_RESIZE_OBSERVER: "",
    NPM_PACKAGE_DEPENDENCIES_SETIMMEDIATE: "",
    NPM_PACKAGE_DEPENDENCIES_WHATWG_FETCH: "",
    NPM_PACKAGE_DEPENDENCIES__WEBCOMPONENTS_SHADYDOM: "",
    NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE: "",
    NPM_PACKAGE_DEPENDENCIES_NODE_PARENTELEMENT: "",
    NPM_PACKAGE_DEPENDENCIES_REGENERATOR_RUNTIME: "",
    NPM_PACKAGE_DEPENDENCIES_PEPJS: "",
    NPM_PACKAGE_DEPENDENCIES_EVENTS_POLYFILL: "",
    NPM_PACKAGE_DEPENDENCIES_MUTATIONOBSERVER_SHIM: "",
    NPM_PACKAGE_DEPENDENCIES_INTL: "",
    NPM_PACKAGE_DEPENDENCIES_INTL_RELATIVE_TIME_FORMAT: "",
    NPM_PACKAGE_DEPENDENCIES_INTL_LIST_FORMAT: "",
    NPM_PACKAGE_DEPENDENCIES__WEBCOMPONENTS_CUSTOM_ELEMENTS: "",
    NPM_PACKAGE_DEPENDENCIES__WESSBERG_POINTER_EVENTS: "",
    NPM_PACKAGE_DEPENDENCIES_CORE_JS: "",
    NPM_PACKAGE_DEPENDENCIES__WEBCOMPONENTS_TEMPLATE: "",
    NPM_PACKAGE_DEPENDENCIES_SYSTEMJS: "",
    NPM_PACKAGE_DEPENDENCIES_WEB_ANIMATIONS_JS: "",
    NPM_PACKAGE_DEPENDENCIES_PROXY_POLYFILL: "",
    NPM_PACKAGE_DEPENDENCIES_PERFNOW: "",
    NPM_PACKAGE_DEPENDENCIES_BLOB_POLYFILL: "",
    NPM_PACKAGE_DEPENDENCIES_REQUESTANIMATIONFRAME: "",
    NPM_PACKAGE_DEPENDENCIES_REQUESTIDLECALLBACK: "",
    NPM_PACKAGE_DEPENDENCIES_URL_POLYFILL: "",
    NPM_PACKAGE_DEPENDENCIES_OBJECT_FIT_IMAGES: "",
    NPM_PACKAGE_DEPENDENCIES_CONSOLE_POLYFILL: "",
    NPM_PACKAGE_DEPENDENCIES_BASE64: "",
    NPM_PACKAGE_DEPENDENCIES_XHR_POLYFILL: "",
    NPM_PACKAGE_DEPENDENCIES_ZONE_JS: "",
    NODE_ENV: "development",
    TESTING: "false",
    DEBUG: "false",
    VERBOSE: "false",
    HTTP2: "true",
    HTTPS: "true",
    CLEAR_CACHE: "false",
    HOST: "0.0.0.0",
    PORT: "3000",
    EMAIL: "",
    KEY: "",
    CERT: ""
};

/**
 * Returns a new object with all of the keys uppercased
 * @param {T} obj
 * @returns {U}
 */
function uppercaseKeys(obj) {
    const newObject = {};
    Object.entries(obj).forEach(([key, value]) => {
        newObject[key.toUpperCase()] = value;
    });
    return newObject;
}

const environment = uppercaseKeys(Object.assign({}, environmentDefaults, process.env));













































// tslint:disable:no-any
/**
 * A class that helps with printing relevant information
 */
class LoggerService {
    constructor(config) {
        /**
         * The prefix to attach to debugging messages
         * @type {string}
         */
        this.DEBUG_PREFIX = chalk.yellow(this.padPrefix("[DEBUG]"));
        /**
         * The prefix to attach to verbose messages
         * @type {string}
         */
        this.VERBOSE_PREFIX = chalk.green(this.padPrefix("[VERBOSE]"));
        /**
         * The prefix to attach to log messages
         * @type {string}
         */
        this.LOG_PREFIX = chalk.magenta(this.padPrefix("[INFO]"));
        /**
         * Whether or not debugging is currently active
         * @type {boolean}
         */
        this._debug = false;
        /**
         * Whether or not verbose output is currently active
         * @type {boolean}
         */
        this._verbose = false;
        this.setDebug(config.debug);
        this.setVerbose(config.verbose);
    }
    /**
     * Sets whether or not debugging is active
     * @param {boolean} debug
     */
    setDebug(debug) {
        this._debug = debug;
    }
    /**
     * Sets whether or not verbose output is active
     * @param {boolean} verbose
     */
    setVerbose(verbose) {
        this._verbose = verbose;
    }
    /**
     * Logs the given message if debugging is activate
     * @param {T[]} messages
     * @returns {T}
     */
    debug(...messages) {
        // Print the message if 'debug' is true
        if (this._debug) {
            console.log(this.DEBUG_PREFIX, ...messages);
        }
        // Return the messages
        return messages;
    }
    /**
     * Logs the given messages
     * @param {T} messages
     * @returns {T[]}
     */
    log(...messages) {
        // Print the message
        console.log(this.LOG_PREFIX, ...messages);
        // Return the messages
        return messages;
    }
    /**
     * Logs the given messages, if verbose output is active
     * @param {T} messages
     * @returns {T[]}
     */
    verbose(...messages) {
        // Print the message if 'verbose' is true
        if (this._verbose) {
            console.log(this.VERBOSE_PREFIX, ...messages);
        }
        // Return the messages
        return messages;
    }
    /**
     * Pads a prefix to nicely align text inside the console
     * @param {string} prefix
     * @returns {string}
     */
    padPrefix(prefix) {
        return prefix.padEnd(10, " ");
    }
    static get [Symbol.for("___CTOR_ARGS___")]() { return ["IConfig"]; }
}

/**
 * A class that helps with compressing files with Brotli and Zlib
 */
class CompressorService {
    constructor(zlibCompressionOptions, brotliCompressionOptions) {
        this.zlibCompressionOptions = zlibCompressionOptions;
        this.brotliCompressionOptions = brotliCompressionOptions;
    }
    /**
     * Compresses the given code based on the given options
     * @param {Buffer} content
     * @returns {Promise<ICompressorServiceCompressResult>}
     */
    async compress(content) {
        return {
            brotli: await this.compressWithBrotli(content),
            zlib: await this.compressWithZlib(content)
        };
    }
    /**
     * Compresses the given code with Brotli based on the given options
     * @param {string} code
     * @returns {Promise<Buffer>}
     */
    async compressWithBrotli(code) {
        return new Promise((resolve, reject) => {
            iltorb.compress(code, this.brotliCompressionOptions, (err, output) => {
                if (err != null)
                    return reject(err);
                return resolve(output);
            });
        });
    }
    /**
     * Compresses the given code with Zlib based on the given options
     * @param {Buffer} code
     * @returns {Promise<Buffer>}
     */
    async compressWithZlib(code) {
        return new Promise((resolve, reject) => {
            zlib.gzip(code, this.zlibCompressionOptions, (err, output) => {
                if (err != null)
                    return reject(err);
                return resolve(output);
            });
        });
    }
    static get [Symbol.for("___CTOR_ARGS___")]() { return ["ZlibOptions", "BrotliEncodeParams"]; }
}

/**
 * A class that helps with minifying code
 */
class MinifyService {
    /**
     * Minifies the given code based on the given options
     * @param {IMinifyServiceOptions} options
     * @returns {string}
     */
    async minify(options) {
        const result = await core.transform(options.code, {
            configFile: false,
            babelrc: false,
            babelrcRoots: false,
            code: true,
            comments: false,
            compact: true,
            filename: "",
            filenameRelative: "",
            sourceMaps: false,
            root: "",
            presets: [],
            plugins: []
        });
        return result.code;
    }
}

/**
 * A service that can flatten the given code into a single file
 */
class FlattenerService {
    /**
     * Flattens the given code into a single file based on the given options
     * @param {IFlattenerOptions} options
     * @return {Promise<string>}
     */
    async flatten({ path, transform }) {
        const paths = Array.isArray(path) ? path : [path];
        if (paths.length < 1)
            return "";
        const bundle = await rollup.rollup({
            input: paths,
            context: "this",
            plugins: [
                multiEntry(),
                {
                    name: "transformer",
                    transform(code, id) {
                        if (transform == null)
                            return;
                        const result = transform.call(this, code, id);
                        if (result == null)
                            return result;
                        return {
                            code: astring.generate(result),
                            map: { mappings: "" }
                        };
                    }
                },
                nodeResolve({
                    module: true,
                    jsnext: true
                }),
                commonjs()
            ]
        });
        const bundleSet = await bundle.generate({
            format: "iife",
            name: paths[0],
            sourcemap: false
        });
        // Produce a flattened bundle
        let flattened = bundleSet.output.map(file => file.code).join("\n");
        // If the IIFE is named, make sure to remove its outer declaration. We only care about the side-effects
        const indexOfIife = flattened.indexOf("(function");
        if (indexOfIife >= 0) {
            flattened = flattened.slice(indexOfIife);
        }
        return flattened;
    }
}



/**
 * Converts the given string to a boolean
 * @param {string} str
 * @returns {boolean}
 */
function booleanize(str) {
    switch (str.toLowerCase().trim()) {
        case "true":
        case "yes":
        case "1":
            return true;
        case "false":
        case "no":
        case "0":
        case null:
            return false;
        default:
            return Boolean(str);
    }
}

/**
 * The separator to use between polyfill names within raw queries
 * @type {string}
 */
const polyfillRawSeparator = ",";

/**
 * The divider to use within raw queries between polyfill names and whether to force-apply them.
 * @type {string}
 */
const polyfillRawDivider = "|";

/**
 * The separator to use between keys and values in raw polyfill queries when parsing options
 * @type {string}
 */
const polyfillOptionKeyValueDivider = "=";

/**
 * The name for the force argument within raw queries, immediately following the polyfill divider
 * @type {string}
 */
const polyfillRawForceName = "force";

/**
 * The separator to use between keys and values in raw polyfill queries when parsing options
 * @type {string}
 */
const polyfillOptionValueSeparator = "~";



// tslint:disable:no-duplicate-string
const constant = {
    endpoint: {
        index: ["/api", "/api/"],
        polyfill: "/api/polyfill"
    },
    header: {
        polyfills: "x-applied-polyfills",
        maxChars: 600
    },
    meta: {
        name: environment.NPM_PACKAGE_NAME,
        version: environment.NPM_PACKAGE_VERSION,
        github: environment.NPM_PACKAGE_HOMEPAGE
    },
    path: {
        cacheRoot: path.join(tempDir, environment.NPM_PACKAGE_NAME),
        cachePackageVersionMap: path.join(tempDir, environment.NPM_PACKAGE_NAME, "cache_package_version_map.json")
    },
    polyfill: {
        systemjs: {
            library: "systemjs",
            relativePaths: ["dist/system.min.js"],
            meta: {
                system: "dist/system.min.js",
                s: "dist/s.min.js"
            },
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_SYSTEMJS,
            dependencies: ["es.promise", "fetch"],
            contexts: ALL_CONTEXTS
        },
        zone: {
            library: "zone.js",
            meta: {
                error: "dist/zone-error.min.js"
            },
            relativePaths: {
                window: ["dist/zone.min.js"],
                worker: ["dist/zone.min.js"],
                node: ["dist/zone-node.js"]
            },
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_ZONE_JS,
            dependencies: [],
            mustComeAfter: "*",
            contexts: WINDOW_NODE_CONTEXT
        },
        "performance.now": {
            library: {
                window: "perfnow",
                worker: "perfnow",
                node: "performance-now"
            },
            relativePaths: {
                window: ["perfnow.min.js"],
                worker: ["perfnow.min.js"],
                node: ["lib/performance-now.js"]
            },
            features: ["high-resolution-time"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_PERFNOW,
            dependencies: ["es.date.now"],
            contexts: ALL_CONTEXTS
        },
        url: {
            library: "url-polyfill",
            relativePaths: ["url-polyfill.min.js"],
            features: ["url", "urlsearchparams"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_URL_POLYFILL,
            dependencies: ["es.object.define-properties", "es.array.for-each"],
            contexts: ALL_CONTEXTS
        },
        "object-fit": {
            localPaths: ["node_modules/object-fit-images/dist/ofi.min.js", "polyfill-lib/object-fit/object-fit-hook.js"],
            features: ["object-fit"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_OBJECT_FIT_IMAGES,
            dependencies: ["get-computed-style", "es.object.define-property"],
            contexts: WINDOW_CONTEXT
        },
        console: {
            library: "console-polyfill",
            relativePaths: ["index.js"],
            features: ["console-basic", "console-time"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CONSOLE_POLYFILL,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        base64: {
            library: "Base64",
            relativePaths: ["base64.js"],
            features: ["atob-btoa"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_BASE64,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        blob: {
            library: "blob-polyfill",
            relativePaths: ["Blob.js"],
            features: ["blobbuilder", "bloburls"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_BLOB_POLYFILL,
            dependencies: ["base64", "url"],
            contexts: ALL_CONTEXTS
        },
        requestidlecallback: {
            polyfills: ["request-idle-callback"]
        },
        requestanimationframe: {
            polyfills: ["request-animation-frame"]
        },
        "request-idle-callback": {
            library: "requestidlecallback",
            relativePaths: ["index.js"],
            features: ["requestidlecallback"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_REQUESTIDLECALLBACK,
            dependencies: ["requestanimationframe"],
            contexts: WINDOW_CONTEXT
        },
        "request-animation-frame": {
            library: "requestanimationframe",
            relativePaths: ["app/requestAnimationFrame.js"],
            features: ["requestanimationframe"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_REQUESTANIMATIONFRAME,
            dependencies: ["es.date.now", "performance.now"],
            contexts: WINDOW_CONTEXT
        },
        proxy: {
            library: "proxy-polyfill",
            relativePaths: ["proxy.min.js"],
            features: ["proxy"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_PROXY_POLYFILL,
            dependencies: ["es"],
            contexts: ALL_CONTEXTS
        },
        es: {
            polyfills: [
                "es.promise",
                "es.object",
                "es.function",
                "es.array",
                "es.array-buffer",
                "es.string",
                "es.data-view",
                "es.regexp",
                "es.number",
                "es.math",
                "es.date",
                "es.symbol",
                "es.collections",
                "es.typed-array",
                "es.reflect"
            ]
        },
        // es2015 is an alias for 'es'
        es2015: {
            polyfills: ["es"]
        },
        "es.promise": {
            polyfills: ["es.promise.constructor", "es.promise.finally"]
        },
        "es.promise.constructor": {
            library: "core-js",
            relativePaths: ["modules/es.promise.js"],
            features: ["promises"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.promise.finally": {
            library: "core-js",
            relativePaths: ["modules/es.promise.finally.js"],
            features: ["javascript.builtins.Promise.finally"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.promise.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.promise.all-settled": {
            library: "core-js",
            relativePaths: ["modules/esnext.promise.all-settled.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.promise.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.promise.try": {
            library: "core-js",
            relativePaths: ["modules/esnext.promise.try.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.promise.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.object": {
            polyfills: [
                "es.object.assign",
                "es.object.create",
                "es.object.define-getter",
                "es.object.define-setter",
                "es.object.entries",
                "es.object.from-entries",
                "es.object.get-own-property-descriptors",
                "es.object.lookup-getter",
                "es.object.lookup-setter",
                "es.object.values",
                "es.object.define-properties",
                "es.object.define-property",
                "es.object.freeze",
                "es.object.get-own-property-descriptor",
                "es.object.get-own-property-names",
                "es.object.get-prototype-of",
                "es.object.is-extensible",
                "es.object.is-frozen",
                "es.object.is-sealed",
                "es.object.is",
                "es.object.keys",
                "es.object.prevent-extensions",
                "es.object.seal",
                "es.object.set-prototype-of",
                "es.object.to-string"
            ]
        },
        "es.object.lookup-getter": {
            library: "core-js",
            relativePaths: ["modules/es.object.lookup-getter.js"],
            features: ["javascript.builtins.Object.lookupGetter"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.from-entries": {
            library: "core-js",
            relativePaths: ["modules/es.object.from-entries.js"],
            features: ["javascript.builtins.Object.fromEntries"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.lookup-setter": {
            library: "core-js",
            relativePaths: ["modules/es.object.lookup-setter.js"],
            features: ["javascript.builtins.Object.lookupSetter"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.define-getter": {
            library: "core-js",
            relativePaths: ["modules/es.object.define-getter.js"],
            features: ["javascript.builtins.Object.defineGetter"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.define-setter": {
            library: "core-js",
            relativePaths: ["modules/es.object.define-setter.js"],
            features: ["javascript.builtins.Object.defineSetter"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.entries": {
            library: "core-js",
            relativePaths: ["modules/es.object.entries.js"],
            features: ["javascript.builtins.Object.entries"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.values": {
            library: "core-js",
            relativePaths: ["modules/es.object.values.js"],
            features: ["object-values"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.get-own-property-descriptors": {
            library: "core-js",
            relativePaths: ["modules/es.object.get-own-property-descriptors.js"],
            features: ["javascript.builtins.Object.getOwnPropertyDescriptors"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.assign": {
            library: "core-js",
            relativePaths: ["modules/es.object.assign.js"],
            features: ["javascript.builtins.Object.assign"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.create": {
            library: "core-js",
            relativePaths: ["modules/es.object.create.js"],
            features: ["javascript.builtins.Object.create"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.define-properties": {
            library: "core-js",
            relativePaths: ["modules/es.object.define-properties.js"],
            features: ["javascript.builtins.Object.defineProperties"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.define-property": {
            library: "core-js",
            relativePaths: ["modules/es.object.define-property.js"],
            features: ["javascript.builtins.Object.defineProperty"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.freeze": {
            library: "core-js",
            relativePaths: ["modules/es.object.freeze.js"],
            features: ["javascript.builtins.Object.freeze"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.get-own-property-descriptor": {
            library: "core-js",
            relativePaths: ["modules/es.object.get-own-property-descriptor.js"],
            features: ["javascript.builtins.Object.getOwnPropertyDescriptor"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.get-own-property-names": {
            library: "core-js",
            relativePaths: ["modules/es.object.get-own-property-names.js"],
            features: ["javascript.builtins.Object.getOwnPropertyNames"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.get-prototype-of": {
            library: "core-js",
            relativePaths: ["modules/es.object.get-prototype-of.js"],
            features: ["javascript.builtins.Object.getPrototypeOf"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.is-extensible": {
            library: "core-js",
            relativePaths: ["modules/es.object.is-extensible.js"],
            features: ["javascript.builtins.Object.isExtensible"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.is-frozen": {
            library: "core-js",
            relativePaths: ["modules/es.object.is-frozen.js"],
            features: ["javascript.builtins.Object.isFrozen"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.is-sealed": {
            library: "core-js",
            relativePaths: ["modules/es.object.is-sealed.js"],
            features: ["javascript.builtins.Object.isSealed"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.is": {
            library: "core-js",
            relativePaths: ["modules/es.object.is.js"],
            features: ["javascript.builtins.Object.is"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.keys": {
            library: "core-js",
            relativePaths: ["modules/es.object.keys.js"],
            features: ["javascript.builtins.Object.keys"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.prevent-extensions": {
            library: "core-js",
            relativePaths: ["modules/es.object.prevent-extensions.js"],
            features: ["javascript.builtins.Object.preventExtensions"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.seal": {
            library: "core-js",
            relativePaths: ["modules/es.object.seal.js"],
            features: ["javascript.builtins.Object.seal"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.set-prototype-of": {
            library: "core-js",
            relativePaths: ["modules/es.object.set-prototype-of.js"],
            features: ["javascript.builtins.Object.setPrototypeOf"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.object.to-string": {
            library: "core-js",
            relativePaths: ["modules/es.object.to-string.js"],
            features: ["javascript.builtins.Object.toString"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.function": {
            polyfills: ["es.function.bind", "es.function.name"]
        },
        "es.function.bind": {
            library: "core-js",
            relativePaths: ["modules/es.function.bind.js"],
            features: ["javascript.builtins.Function.bind"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.function.name": {
            library: "core-js",
            relativePaths: ["modules/es.function.name.js"],
            features: ["javascript.builtins.Function.name"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array": {
            polyfills: [
                "es.array.concat",
                "es.array.copy-within",
                "es.array.every",
                "es.array.flat",
                "es.array.flat-map",
                "es.array.fill",
                "es.array.filter",
                "es.array.find",
                "es.array.find-index",
                "es.array.for-each",
                "es.array.from",
                "es.array.includes",
                "es.array.index-of",
                "es.array.is-array",
                "es.array.iterator",
                "es.array.join",
                "es.array.last-index-of",
                "es.array.map",
                "es.array.of",
                "es.array.reduce",
                "es.array.reduce-right",
                "es.array.slice",
                "es.array.some",
                "es.array.sort",
                "es.array.species",
                "es.array.splice"
            ]
        },
        "es.array.concat": {
            library: "core-js",
            relativePaths: ["modules/es.array.concat.js"],
            features: ["javascript.builtins.Array.concat"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.flat": {
            library: "core-js",
            relativePaths: ["modules/es.array.flat.js"],
            features: ["javascript.builtins.Array.flat"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.flat-map": {
            library: "core-js",
            relativePaths: ["modules/es.array.flat-map.js"],
            features: ["javascript.builtins.Array.flatMap"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.last-index": {
            library: "core-js",
            relativePaths: ["modules/esnext.array.last-index.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.last-item": {
            library: "core-js",
            relativePaths: ["modules/esnext.array.last-item.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.copy-within": {
            library: "core-js",
            relativePaths: ["modules/es.array.copy-within.js"],
            features: ["javascript.builtins.Array.copyWithin"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.every": {
            library: "core-js",
            relativePaths: ["modules/es.array.every.js"],
            features: ["javascript.builtins.Array.every"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.fill": {
            library: "core-js",
            relativePaths: ["modules/es.array.fill.js"],
            features: ["javascript.builtins.Array.fill"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.filter": {
            library: "core-js",
            relativePaths: ["modules/es.array.filter.js"],
            features: ["javascript.builtins.Array.filter"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.find-index": {
            library: "core-js",
            relativePaths: ["modules/es.array.find-index.js"],
            features: ["javascript.builtins.Array.findIndex"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.find": {
            library: "core-js",
            relativePaths: ["modules/es.array.find.js"],
            features: ["javascript.builtins.Array.find"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.for-each": {
            library: "core-js",
            relativePaths: ["modules/es.array.for-each.js"],
            features: ["javascript.builtins.Array.forEach"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.from": {
            library: "core-js",
            relativePaths: ["modules/es.array.from.js"],
            features: ["javascript.builtins.Array.from"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.includes": {
            library: "core-js",
            relativePaths: ["modules/es.array.includes.js"],
            features: ["javascript.builtins.Array.includes"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.index-of": {
            library: "core-js",
            relativePaths: ["modules/es.array.index-of.js"],
            features: ["javascript.builtins.Array.indexOf"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.is-array": {
            library: "core-js",
            relativePaths: ["modules/es.array.is-array.js"],
            features: ["javascript.builtins.Array.isArray"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.iterator": {
            library: "core-js",
            relativePaths: ["modules/es.array.iterator.js"],
            features: ["javascript.builtins.Array.@@iterator"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.iterator"],
            contexts: ALL_CONTEXTS
        },
        "es.array.join": {
            library: "core-js",
            relativePaths: ["modules/es.array.join.js"],
            features: ["javascript.builtins.Array.join"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.last-index-of": {
            library: "core-js",
            relativePaths: ["modules/es.array.last-index-of.js"],
            features: ["javascript.builtins.Array.lastIndexOf"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.map": {
            library: "core-js",
            relativePaths: ["modules/es.array.map.js"],
            features: ["javascript.builtins.Array.map"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.of": {
            library: "core-js",
            relativePaths: ["modules/es.array.of.js"],
            features: ["javascript.builtins.Array.of"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.reduce-right": {
            library: "core-js",
            relativePaths: ["modules/es.array.reduce-right.js"],
            features: ["javascript.builtins.Array.reduceRight"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.reduce": {
            library: "core-js",
            relativePaths: ["modules/es.array.reduce.js"],
            features: ["javascript.builtins.Array.reduce"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.slice": {
            library: "core-js",
            relativePaths: ["modules/es.array.slice.js"],
            features: ["javascript.builtins.Array.slice"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.some": {
            library: "core-js",
            relativePaths: ["modules/es.array.some.js"],
            features: ["javascript.builtins.Array.some"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.sort": {
            library: "core-js",
            relativePaths: ["modules/es.array.sort.js"],
            features: ["javascript.builtins.Array.sort"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array.species": {
            library: "core-js",
            relativePaths: ["modules/es.array.species.js"],
            features: ["javascript.builtins.Array.@@species"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.species"],
            contexts: ALL_CONTEXTS
        },
        "es.array.splice": {
            library: "core-js",
            relativePaths: ["modules/es.array.splice.js"],
            features: ["javascript.builtins.Array.splice"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array-buffer": {
            polyfills: ["es.array-buffer.constructor", "es.array-buffer.is-view", "es.array-buffer.slice"]
        },
        "es.array-buffer.constructor": {
            library: "core-js",
            relativePaths: ["modules/es.array-buffer.constructor.js"],
            features: ["javascript.builtins.ArrayBuffer"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.array-buffer.is-view": {
            library: "core-js",
            relativePaths: ["modules/es.array-buffer.is-view.js"],
            features: ["javascript.builtins.ArrayBuffer.isView"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.array-buffer.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.array-buffer.slice": {
            library: "core-js",
            relativePaths: ["modules/es.array-buffer.slice.js"],
            features: ["javascript.builtins.ArrayBuffer.slice"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.array-buffer.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.string": {
            polyfills: [
                "es.string.anchor",
                "es.string.big",
                "es.string.blink",
                "es.string.bold",
                "es.string.code-point-at",
                "es.string.ends-with",
                "es.string.fixed",
                "es.string.fontcolor",
                "es.string.fontsize",
                "es.string.from-code-point",
                "es.string.includes",
                "es.string.italics",
                "es.string.iterator",
                "es.string.link",
                "es.string.match",
                "es.string.pad-end",
                "es.string.pad-start",
                "es.string.raw",
                "es.string.repeat",
                "es.string.search",
                "es.string.small",
                "es.string.split",
                "es.string.starts-with",
                "es.string.strike",
                "es.string.sub",
                "es.string.sup",
                "es.string.trim",
                "es.string.trim-start",
                "es.string.trim-end"
            ]
        },
        "es.string.at": {
            library: "core-js",
            relativePaths: ["modules/esnext.string.at.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.code-points": {
            library: "core-js",
            relativePaths: ["modules/esnext.string.code-points.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.match-all": {
            library: "core-js",
            relativePaths: ["modules/esnext.string.match-all.js"],
            features: ["javascript.builtins.String.matchAll"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.replace-all": {
            library: "core-js",
            relativePaths: ["modules/esnext.string.replace-all.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.trim-start": {
            library: "core-js",
            relativePaths: ["modules/es.string.trim-start.js"],
            features: ["javascript.builtins.String.trimStart"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.trim-end": {
            library: "core-js",
            relativePaths: ["modules/es.string.trim-end.js"],
            features: ["javascript.builtins.String.trimEnd"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.anchor": {
            library: "core-js",
            relativePaths: ["modules/es.string.anchor.js"],
            features: ["javascript.builtins.String.anchor"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.big": {
            library: "core-js",
            relativePaths: ["modules/es.string.big.js"],
            features: ["javascript.builtins.String.big"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.blink": {
            library: "core-js",
            relativePaths: ["modules/es.string.blink.js"],
            features: ["javascript.builtins.String.blink"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.bold": {
            library: "core-js",
            relativePaths: ["modules/es.string.bold.js"],
            features: ["javascript.builtins.String.bold"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.code-point-at": {
            library: "core-js",
            relativePaths: ["modules/es.string.code-point-at.js"],
            features: ["javascript.builtins.String.codePointAt"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.ends-with": {
            library: "core-js",
            relativePaths: ["modules/es.string.ends-with.js"],
            features: ["javascript.builtins.String.endsWith"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.fixed": {
            library: "core-js",
            relativePaths: ["modules/es.string.fixed.js"],
            features: ["javascript.builtins.String.fixed"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.fontcolor": {
            library: "core-js",
            relativePaths: ["modules/es.string.fontcolor.js"],
            features: ["javascript.builtins.String.fontcolor"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.fontsize": {
            library: "core-js",
            relativePaths: ["modules/es.string.fontsize.js"],
            features: ["javascript.builtins.String.fontsize"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.from-code-point": {
            library: "core-js",
            relativePaths: ["modules/es.string.from-code-point.js"],
            features: ["javascript.builtins.String.fromCodePoint"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.includes": {
            library: "core-js",
            relativePaths: ["modules/es.string.includes.js"],
            features: ["javascript.builtins.String.includes"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.italics": {
            library: "core-js",
            relativePaths: ["modules/es.string.italics.js"],
            features: ["javascript.builtins.String.italics"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.iterator": {
            library: "core-js",
            relativePaths: ["modules/es.string.iterator.js"],
            features: ["javascript.builtins.String.@@iterator"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.iterator"],
            contexts: ALL_CONTEXTS
        },
        "es.string.link": {
            library: "core-js",
            relativePaths: ["modules/es.string.link.js"],
            features: ["javascript.builtins.String.link"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.match": {
            library: "core-js",
            relativePaths: ["modules/es.string.match.js"],
            features: ["javascript.builtins.String.match"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.pad-end": {
            library: "core-js",
            relativePaths: ["modules/es.string.pad-end.js"],
            features: ["javascript.builtins.String.padEnd"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.pad-start": {
            library: "core-js",
            relativePaths: ["modules/es.string.pad-start.js"],
            features: ["javascript.builtins.String.padStart"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.raw": {
            library: "core-js",
            relativePaths: ["modules/es.string.raw.js"],
            features: ["javascript.builtins.String.raw"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.repeat": {
            library: "core-js",
            relativePaths: ["modules/es.string.repeat.js"],
            features: ["javascript.builtins.String.repeat"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.replace": {
            library: "core-js",
            relativePaths: ["modules/es.string.replace.js"],
            features: ["javascript.builtins.String.replace"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.search": {
            library: "core-js",
            relativePaths: ["modules/es.string.search.js"],
            features: ["javascript.builtins.String.search"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.small": {
            library: "core-js",
            relativePaths: ["modules/es.string.small.js"],
            features: ["javascript.builtins.String.small"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.split": {
            library: "core-js",
            relativePaths: ["modules/es.string.split.js"],
            features: ["javascript.builtins.String.split"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.starts-with": {
            library: "core-js",
            relativePaths: ["modules/es.string.starts-with.js"],
            features: ["javascript.builtins.String.startsWith"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.strike": {
            library: "core-js",
            relativePaths: ["modules/es.string.strike.js"],
            features: ["javascript.builtins.String.strike"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.sub": {
            library: "core-js",
            relativePaths: ["modules/es.string.sub.js"],
            features: ["javascript.builtins.String.sub"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.sup": {
            library: "core-js",
            relativePaths: ["modules/es.string.sup.js"],
            features: ["javascript.builtins.String.sup"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.string.trim": {
            library: "core-js",
            relativePaths: ["modules/es.string.trim.js"],
            features: ["javascript.builtins.String.trim"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.regexp": {
            polyfills: ["es.regexp.constructor", "es.regexp.flags", "es.regexp.to-string"]
        },
        "es.regexp.constructor": {
            library: "core-js",
            relativePaths: ["modules/es.regexp.constructor.js"],
            features: ["javascript.builtins.RegExp"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.regexp.flags": {
            library: "core-js",
            relativePaths: ["modules/es.regexp.flags.js"],
            features: ["javascript.builtins.RegExp.flags"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.regexp.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.regexp.to-string": {
            library: "core-js",
            relativePaths: ["modules/es.regexp.to-string.js"],
            features: ["javascript.builtins.RegExp.toString"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.regexp.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.number": {
            polyfills: [
                "es.number.constructor",
                "es.number.epsilon",
                "es.number.is-finite",
                "es.number.is-integer",
                "es.number.is-nan",
                "es.number.is-safe-integer",
                "es.number.max-safe-integer",
                "es.number.min-safe-integer",
                "es.number.parse-float",
                "es.number.parse-int",
                "es.number.to-fixed",
                "es.number.to-precision"
            ]
        },
        "es.number.from-string": {
            library: "core-js",
            relativePaths: ["modules/esnext.number.from-string.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.constructor": {
            library: "core-js",
            relativePaths: ["modules/es.number.constructor.js"],
            features: ["javascript.builtins.Number.prototype"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.epsilon": {
            library: "core-js",
            relativePaths: ["modules/es.number.epsilon.js"],
            features: ["javascript.builtins.Number.EPSILON"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.is-finite": {
            library: "core-js",
            relativePaths: ["modules/es.number.is-finite.js"],
            features: ["javascript.builtins.Number.isFinite"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.is-integer": {
            library: "core-js",
            relativePaths: ["modules/es.number.is-integer.js"],
            features: ["javascript.builtins.Number.isInteger"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.is-nan": {
            library: "core-js",
            relativePaths: ["modules/es.number.is-nan.js"],
            features: ["javascript.builtins.Number.isNaN"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.is-safe-integer": {
            library: "core-js",
            relativePaths: ["modules/es.number.is-safe-integer.js"],
            features: ["javascript.builtins.Number.isSafeInteger"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.max-safe-integer": {
            library: "core-js",
            relativePaths: ["modules/es.number.max-safe-integer.js"],
            features: ["javascript.builtins.Number.MAX_SAFE_INTEGER"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.min-safe-integer": {
            library: "core-js",
            relativePaths: ["modules/es.number.min-safe-integer.js"],
            features: ["javascript.builtins.Number.MIN_SAFE_INTEGER"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.parse-float": {
            library: "core-js",
            relativePaths: ["modules/es.number.parse-float.js"],
            features: ["javascript.builtins.Number.parseFloat"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.parse-int": {
            library: "core-js",
            relativePaths: ["modules/es.number.parse-int.js"],
            features: ["javascript.builtins.Number.parseInt"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.to-fixed": {
            library: "core-js",
            relativePaths: ["modules/es.number.to-fixed.js"],
            features: ["javascript.builtins.Number.toFixed"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.number.to-precision": {
            library: "core-js",
            relativePaths: ["modules/es.number.to-precision.js"],
            features: ["javascript.builtins.Number.toPrecision"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math": {
            polyfills: [
                "es.math.acosh",
                "es.math.asinh",
                "es.math.atanh",
                "es.math.cbrt",
                "es.math.clz32",
                "es.math.cosh",
                "es.math.expm1",
                "es.math.fround",
                "es.math.hypot",
                "es.math.imul",
                "es.math.log1p",
                "es.math.log2",
                "es.math.log10",
                "es.math.sign",
                "es.math.sinh",
                "es.math.tanh",
                "es.math.trunc"
            ]
        },
        "es.math.clamp": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.clamp.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.deg-per-rad": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.deg-per-rad.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.degrees": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.degrees.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.fscale": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.fscale.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.iaddh": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.iaddh.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.imulh": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.imulh.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.isubh": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.isubh.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.rad-per-deg": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.rad-per-deg.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.radians": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.radians.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.scale": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.scale.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.signbit": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.signbit.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.umulh": {
            library: "core-js",
            relativePaths: ["modules/esnext.math.umulh.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.acosh": {
            library: "core-js",
            relativePaths: ["modules/es.math.acosh.js"],
            features: ["javascript.builtins.Math.acosh"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.asinh": {
            library: "core-js",
            relativePaths: ["modules/es.math.asinh.js"],
            features: ["javascript.builtins.Math.asinh"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.atanh": {
            library: "core-js",
            relativePaths: ["modules/es.math.atanh.js"],
            features: ["javascript.builtins.Math.atanh"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.cbrt": {
            library: "core-js",
            relativePaths: ["modules/es.math.cbrt.js"],
            features: ["javascript.builtins.Math.cbrt"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.clz32": {
            library: "core-js",
            relativePaths: ["modules/es.math.clz32.js"],
            features: ["javascript.builtins.Math.clz32"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.cosh": {
            library: "core-js",
            relativePaths: ["modules/es.math.cosh.js"],
            features: ["javascript.builtins.Math.cosh"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.expm1": {
            library: "core-js",
            relativePaths: ["modules/es.math.expm1.js"],
            features: ["javascript.builtins.Math.expm1"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.fround": {
            library: "core-js",
            relativePaths: ["modules/es.math.fround.js"],
            features: ["javascript.builtins.Math.fround"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.hypot": {
            library: "core-js",
            relativePaths: ["modules/es.math.hypot.js"],
            features: ["javascript.builtins.Math.hypot"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.imul": {
            library: "core-js",
            relativePaths: ["modules/es.math.imul.js"],
            features: ["javascript.builtins.Math.imul"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.log1p": {
            library: "core-js",
            relativePaths: ["modules/es.math.log1p.js"],
            features: ["javascript.builtins.Math.log1p"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.log2": {
            library: "core-js",
            relativePaths: ["modules/es.math.log2.js"],
            features: ["javascript.builtins.Math.log2"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.log10": {
            library: "core-js",
            relativePaths: ["modules/es.math.log10.js"],
            features: ["javascript.builtins.Math.log10"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.sign": {
            library: "core-js",
            relativePaths: ["modules/es.math.sign.js"],
            features: ["javascript.builtins.Math.sign"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.sinh": {
            library: "core-js",
            relativePaths: ["modules/es.math.sinh.js"],
            features: ["javascript.builtins.Math.sinh"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.tanh": {
            library: "core-js",
            relativePaths: ["modules/es.math.tanh.js"],
            features: ["javascript.builtins.Math.tanh"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.math.trunc": {
            library: "core-js",
            relativePaths: ["modules/es.math.trunc.js"],
            features: ["javascript.builtins.Math.trunc"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.data-view": {
            library: "core-js",
            relativePaths: ["modules/es.data-view.js"],
            features: ["javascript.builtins.DataView"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.date": {
            polyfills: ["es.date.now", "es.date.to-iso-string", "es.date.to-json", "es.date.to-primitive", "es.date.to-string"]
        },
        "es.date.now": {
            library: "core-js",
            relativePaths: ["modules/es.date.now.js"],
            features: ["javascript.builtins.Date.now"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.date.to-iso-string": {
            library: "core-js",
            relativePaths: ["modules/es.date.to-iso-string.js"],
            features: ["javascript.builtins.Date.toISOString"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.date.to-json": {
            library: "core-js",
            relativePaths: ["modules/es.date.to-json.js"],
            features: ["javascript.builtins.Date.toJSON"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.date.to-primitive": {
            library: "core-js",
            relativePaths: ["modules/es.date.to-primitive.js"],
            features: ["javascript.builtins.Date.@@toPrimitive"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.to-primitive"],
            contexts: ALL_CONTEXTS
        },
        "es.date.to-string": {
            library: "core-js",
            relativePaths: ["modules/es.date.to-string.js"],
            features: ["javascript.builtins.Date.toString"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.symbol": {
            polyfills: [
                "es.symbol.async-iterator",
                "es.symbol.has-instance",
                "es.symbol.is-concat-spreadable",
                "es.symbol.iterator",
                "es.symbol.constructor",
                "es.symbol.match",
                "es.symbol.replace",
                "es.symbol.search",
                "es.symbol.species",
                "es.symbol.split",
                "es.symbol.to-primitive",
                "es.symbol.to-string-tag",
                "es.symbol.unscopables"
            ]
        },
        "es.symbol.description": {
            library: "core-js",
            relativePaths: ["modules/esnext.symbol.description.js"],
            features: ["javascript.builtins.Symbol.description"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.pattern-match": {
            library: "core-js",
            relativePaths: ["modules/esnext.symbol.pattern-match.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.constructor": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.js"],
            features: ["javascript.builtins.Symbol"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.async-iterator": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.async-iterator.js"],
            features: ["javascript.builtins.Symbol.asyncIterator"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.has-instance": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.has-instance.js", "modules/es.function.has-instance.js"],
            features: ["javascript.builtins.Symbol.hasInstance"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.is-concat-spreadable": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.is-concat-spreadable.js"],
            features: ["javascript.builtins.Symbol.isConcatSpreadable"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.iterator": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.iterator.js"],
            features: ["javascript.builtins.Symbol.iterator"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.match": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.match.js"],
            features: ["javascript.builtins.Symbol.match"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.replace": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.replace.js"],
            features: ["javascript.builtins.Symbol.replace"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.search": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.search.js"],
            features: ["javascript.builtins.Symbol.search"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.species": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.species.js"],
            features: ["javascript.builtins.Symbol.species"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.split": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.split.js"],
            features: ["javascript.builtins.Symbol.split"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.to-primitive": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.to-primitive.js"],
            features: ["javascript.builtins.Symbol.toPrimitive"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.to-string-tag": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.to-string-tag.js", "modules/es.json.to-string-tag.js", "modules/es.math.to-string-tag.js"],
            features: ["javascript.builtins.Symbol.toStringTag"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.symbol.unscopables": {
            library: "core-js",
            relativePaths: ["modules/es.symbol.unscopables.js"],
            features: ["javascript.builtins.Symbol.unscopables"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.constructor"],
            contexts: ALL_CONTEXTS
        },
        "es.collections": {
            polyfills: ["es.map", "es.weak-map", "es.set", "es.weak-set"]
        },
        "es.map": {
            library: "core-js",
            relativePaths: ["modules/es.map.js"],
            features: ["javascript.builtins.Map", "javascript.builtins.Map.@@iterator"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.map.every": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.every.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.filter": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.filter.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.find": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.find.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.find-key": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.find-key.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.from": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.from.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.group-by": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.group-by.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.includes": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.includes.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.key-by": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.key-by.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.key-of": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.key-of.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.map-keys": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.map-keys.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.map-values": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.map-values.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.merge": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.merge.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.of": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.of.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.reduce": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.reduce.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.some": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.some.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.map.update": {
            library: "core-js",
            relativePaths: ["modules/esnext.map.update.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.map"],
            contexts: ALL_CONTEXTS
        },
        "es.weak-map": {
            library: "core-js",
            relativePaths: ["modules/es.weak-map.js"],
            features: ["javascript.builtins.WeakMap"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.weak-map.from": {
            library: "core-js",
            relativePaths: ["modules/esnext.weak-map.from.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.weak-map"],
            contexts: ALL_CONTEXTS
        },
        "es.weak-map.of": {
            library: "core-js",
            relativePaths: ["modules/esnext.weak-map.of.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.weak-map"],
            contexts: ALL_CONTEXTS
        },
        "es.set": {
            library: "core-js",
            relativePaths: ["modules/es.set.js"],
            features: ["javascript.builtins.Set", "javascript.builtins.Set.@@iterator"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.set.add-all": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.addTime-all.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.delete-all": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.delete-all.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.difference": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.difference.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.every": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.every.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.filter": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.filter.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.find": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.find.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.from": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.from.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.intersection": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.intersection.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.join": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.join.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.map": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.map.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.of": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.of.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.reduce": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.reduce.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.some": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.some.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.symmetric-difference": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.symmetric-difference.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.union": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.union.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.is-disjoint-from": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.is-disjoint-from.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.is-subset-of": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.is-subset-of.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.set.is-superset-of": {
            library: "core-js",
            relativePaths: ["modules/esnext.set.is-superset-of.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.set"],
            contexts: ALL_CONTEXTS
        },
        "es.weak-set": {
            library: "core-js",
            relativePaths: ["modules/es.weak-set.js"],
            features: ["javascript.builtins.WeakSet"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.weak-set.from": {
            library: "core-js",
            relativePaths: ["modules/esnext.weak-set.from.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.weak-set"],
            contexts: ALL_CONTEXTS
        },
        "es.weak-set.of": {
            library: "core-js",
            relativePaths: ["modules/esnext.weak-set.of.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.weak-set"],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array": {
            polyfills: [
                "es.typed-array.copy-within",
                "es.typed-array.every",
                "es.typed-array.fill",
                "es.typed-array.filter",
                "es.typed-array.find",
                "es.typed-array.find-index",
                "es.typed-array.float32-array",
                "es.typed-array.float64-array",
                "es.typed-array.for-each",
                "es.typed-array.from",
                "es.typed-array.includes",
                "es.typed-array.index-of",
                "es.typed-array.int8-array",
                "es.typed-array.int16-array",
                "es.typed-array.int32-array",
                "es.typed-array.iterator",
                "es.typed-array.join",
                "es.typed-array.last-index-of",
                "es.typed-array.map",
                "es.typed-array.of",
                "es.typed-array.reduce",
                "es.typed-array.reduce-right",
                "es.typed-array.reverse",
                "es.typed-array.set",
                "es.typed-array.slice",
                "es.typed-array.some",
                "es.typed-array.sort",
                "es.typed-array.subarray",
                "es.typed-array.to-locale-string",
                "es.typed-array.to-string",
                "es.typed-array.uint8-array",
                "es.typed-array.uint8-clamped-array",
                "es.typed-array.uint16-array",
                "es.typed-array.uint32-array"
            ]
        },
        "es.typed-array.copy-within": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.copy-within.js"],
            features: ["javascript.builtins.TypedArray.copyWithin"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.every": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.every.js"],
            features: ["javascript.builtins.TypedArray.every"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.fill": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.fill.js"],
            features: ["javascript.builtins.TypedArray.fill"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.filter": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.filter.js"],
            features: ["javascript.builtins.TypedArray.filter"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.find": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.find.js"],
            features: ["javascript.builtins.TypedArray.find"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.find-index": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.find-index.js"],
            features: ["javascript.builtins.TypedArray.findIndex"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.float32-array": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.float32-array.js"],
            features: ["javascript.builtins.Float32Array"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.float64-array": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.float64-array.js"],
            features: ["javascript.builtins.Float64Array"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.for-each": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.for-each.js"],
            features: ["javascript.builtins.TypedArray.forEach"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.from": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.from.js"],
            features: ["javascript.builtins.TypedArray.from"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.includes": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.includes.js"],
            features: ["javascript.builtins.TypedArray.includes"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.index-of": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.index-of.js"],
            features: ["javascript.builtins.TypedArray.indexOf"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.int8-array": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.int8-array.js"],
            features: ["javascript.builtins.Int8Array"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.int16-array": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.int16-array.js"],
            features: ["javascript.builtins.Int16Array"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.int32-array": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.int32-array.js"],
            features: ["javascript.builtins.Int32Array"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.iterator": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.iterator.js"],
            features: ["javascript.builtins.TypedArray.@@iterator"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.iterator"],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.join": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.join.js"],
            features: ["javascript.builtins.TypedArray.join"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.last-index-of": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.last-index-of.js"],
            features: ["javascript.builtins.TypedArray.lastIndexOf"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.map": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.map.js"],
            features: ["javascript.builtins.TypedArray.map"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.of": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.of.js"],
            features: ["javascript.builtins.TypedArray.of"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.reduce": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.reduce.js"],
            features: ["javascript.builtins.TypedArray.reduce"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.reduce-right": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.reduce-right.js"],
            features: ["javascript.builtins.TypedArray.reduceRight"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.reverse": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.reverse.js"],
            features: ["javascript.builtins.TypedArray.reverse"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.set": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.set.js"],
            features: ["javascript.builtins.TypedArray.set"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.slice": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.slice.js"],
            features: ["javascript.builtins.TypedArray.slice"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.some": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.some.js"],
            features: ["javascript.builtins.TypedArray.some"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.sort": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.sort.js"],
            features: ["javascript.builtins.TypedArray.sort"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.subarray": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.subarray.js"],
            features: ["javascript.builtins.TypedArray.subarray"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.to-locale-string": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.to-locale-string.js"],
            features: ["javascript.builtins.TypedArray.toLocaleString"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.to-string": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.to-string.js"],
            features: ["javascript.builtins.TypedArray.toString"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.uint8-array": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.uint8-array.js"],
            features: ["javascript.builtins.Uint8Array"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.uint8-clamped-array": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.uint8-clamped-array.js"],
            features: ["javascript.builtins.Uint8ClampedArray"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.uint16-array": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.uint16-array.js"],
            features: ["javascript.builtins.Uint16Array"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.typed-array.uint32-array": {
            library: "core-js",
            relativePaths: ["modules/es.typed-array.uint32-array.js"],
            features: ["javascript.builtins.Uint32Array"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect": {
            polyfills: [
                "es.reflect.apply",
                "es.reflect.construct",
                "es.reflect.define-property",
                "es.reflect.delete-property",
                "es.reflect.get",
                "es.reflect.get-own-property-descriptor",
                "es.reflect.get-prototype-of",
                "es.reflect.has",
                "es.reflect.is-extensible",
                "es.reflect.own-keys",
                "es.reflect.prevent-extensions",
                "es.reflect.set",
                "es.reflect.set-prototype-of"
            ]
        },
        "es.reflect.define-metadata": {
            library: "core-js",
            relativePaths: ["modules/esnext.reflect.define-metadata.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.delete-metadata": {
            library: "core-js",
            relativePaths: ["modules/esnext.reflect.delete-metadata.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.get-metadata": {
            library: "core-js",
            relativePaths: ["modules/esnext.reflect.get-metadata.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.get-metadata-keys": {
            library: "core-js",
            relativePaths: ["modules/esnext.reflect.get-metadata-keys.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.get-own-metadata": {
            library: "core-js",
            relativePaths: ["modules/esnext.reflect.get-own-metadata.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.get-own-metadata-keys": {
            library: "core-js",
            relativePaths: ["modules/esnext.reflect.get-own-metadata-keys.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.has-metadata": {
            library: "core-js",
            relativePaths: ["modules/esnext.reflect.has-metadata.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.has-own-metadata": {
            library: "core-js",
            relativePaths: ["modules/esnext.reflect.has-own-metadata.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.metadata": {
            library: "core-js",
            relativePaths: ["modules/esnext.reflect.metadata.js"],
            // TODO: Update when MDN or Caniuse Compatibility is added
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.apply": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.apply.js"],
            features: ["javascript.builtins.Reflect.apply"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.construct": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.construct.js"],
            features: ["javascript.builtins.Reflect.construct"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.define-property": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.define-property.js"],
            features: ["javascript.builtins.Reflect.defineProperty"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.delete-property": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.delete-property.js"],
            features: ["javascript.builtins.Reflect.deleteProperty"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.get": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.get.js"],
            features: ["javascript.builtins.Reflect.get"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.get-own-property-descriptor": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.get-own-property-descriptor.js"],
            features: ["javascript.builtins.Reflect.getOwnPropertyDescriptor"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.get-prototype-of": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.get-prototype-of.js"],
            features: ["javascript.builtins.Reflect.getPrototypeOf"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.has": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.has.js"],
            features: ["javascript.builtins.Reflect.has"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.is-extensible": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.is-extensible.js"],
            features: ["javascript.builtins.Reflect.isExtensible"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.own-keys": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.own-keys.js"],
            features: ["javascript.builtins.Reflect.ownKeys"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.prevent-extensions": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.prevent-extensions.js"],
            features: ["javascript.builtins.Reflect.preventExtensions"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.set": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.set.js"],
            features: ["javascript.builtins.Reflect.set"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "es.reflect.set-prototype-of": {
            library: "core-js",
            relativePaths: ["modules/es.reflect.set-prototype-of.js"],
            features: ["javascript.builtins.Reflect.setPrototypeOf"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        esnext: {
            polyfills: ["esnext.array", "esnext.collections", "esnext.math", "esnext.number", "esnext.object", "esnext.promise", "esnext.reflect", "esnext.string", "esnext.symbol"]
        },
        // An alias for the alias 'esnext'
        "es2016+": {
            polyfills: ["esnext"]
        },
        "esnext.array": {
            polyfills: ["es.array.last-index", "es.array.last-item"]
        },
        "esnext.object": {
            polyfills: []
        },
        "esnext.collections": {
            polyfills: ["esnext.map", "esnext.weak-map", "esnext.set", "esnext.weak-set"]
        },
        "esnext.map": {
            polyfills: [
                "es.map.every",
                "es.map.filter",
                "es.map.find",
                "es.map.find-key",
                "es.map.from",
                "es.map.group-by",
                "es.map.includes",
                "es.map.key-by",
                "es.map.key-of",
                "es.map.map-keys",
                "es.map.map-values",
                "es.map.merge",
                "es.map.of",
                "es.map.reduce",
                "es.map.some",
                "es.map.update"
            ]
        },
        "esnext.weak-map": {
            polyfills: ["es.weak-map.from", "es.weak-map.of"]
        },
        "esnext.set": {
            polyfills: [
                "es.set.add-all",
                "es.set.delete-all",
                "es.set.difference",
                "es.set.every",
                "es.set.filter",
                "es.set.find",
                "es.set.from",
                "es.set.intersection",
                "es.set.join",
                "es.set.map",
                "es.set.of",
                "es.set.reduce",
                "es.set.some",
                "es.set.symmetric-difference",
                "es.set.union",
                "es.set.is-disjoint-from",
                "es.set.is-subset-of",
                "es.set.is-superset-of"
            ]
        },
        "esnext.weak-set": {
            polyfills: ["es.weak-set.from", "es.weak-set.of"]
        },
        "esnext.math": {
            polyfills: [
                "es.math.clamp",
                "es.math.deg-per-rad",
                "es.math.degrees",
                "es.math.fscale",
                "es.math.iaddh",
                "es.math.imulh",
                "es.math.isubh",
                "es.math.rad-per-deg",
                "es.math.radians",
                "es.math.scale",
                "es.math.signbit",
                "es.math.umulh"
            ]
        },
        "esnext.number": {
            polyfills: ["es.number.from-string"]
        },
        "esnext.promise": {
            polyfills: ["es.promise.all-settled", "es.promise.try"]
        },
        "esnext.reflect": {
            polyfills: [
                "es.reflect.define-metadata",
                "es.reflect.delete-metadata",
                "es.reflect.get-metadata",
                "es.reflect.get-metadata-keys",
                "es.reflect.get-own-metadata",
                "es.reflect.get-own-metadata-keys",
                "es.reflect.has-metadata",
                "es.reflect.has-own-metadata",
                "es.reflect.metadata"
            ]
        },
        "esnext.string": {
            polyfills: ["es.string.at", "es.string.code-points", "es.string.match-all", "es.string.replace-all"]
        },
        "esnext.symbol": {
            polyfills: ["es.symbol.description", "es.symbol.pattern-match"]
        },
        "dom.collections.iterable": {
            polyfills: ["dom.collections.iterator", "dom.collections.for-each"]
        },
        "dom.collections.iterator": {
            library: "core-js",
            relativePaths: ["modules/web.dom-collections.iterator.js"],
            features: ["api.NodeList.forEach"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.iterator"],
            contexts: WINDOW_CONTEXT
        },
        "dom.collections.for-each": {
            library: "core-js",
            relativePaths: ["modules/web.dom-collections.for-each.js"],
            features: ["api.NodeList.forEach"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: ["es.symbol.iterator"],
            contexts: WINDOW_CONTEXT
        },
        "pointer-event": {
            library: "@wessberg/pointer-events",
            relativePaths: ["dist/index.js"],
            features: ["pointer"],
            version: environment.NPM_PACKAGE_DEPENDENCIES__WESSBERG_POINTER_EVENTS,
            dependencies: [
                // TODO: Also relies on "elementFromPoint" which there isn't a polyfill for yet. Add it to the dependencies when the polyfill is ready
                "es.array.from",
                "es.array.some",
                "es.array.every",
                "es.string.includes",
                "es.set",
                "es.map",
                "es.object.define-properties",
                "es.object.define-property",
                "event",
                "custom-event",
                "get-computed-style"
            ],
            contexts: WINDOW_CONTEXT
        },
        xhr: {
            library: "xhr-polyfill",
            relativePaths: ["dist/xhr-polyfill.js"],
            features: ["xhr2"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_XHR_POLYFILL,
            dependencies: [],
            contexts: WINDOW_CONTEXT
        },
        fetch: {
            localPaths: ["polyfill-lib/fetch/fetch.js"],
            features: ["fetch"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_WHATWG_FETCH,
            dependencies: ["es.array.for-each", "es.object.get-own-property-names", "es.promise", "xhr"],
            contexts: WINDOW_CONTEXT
        },
        intl: {
            polyfills: ["intl.core", "intl.list-format", "intl.relative-time-format"]
        },
        "intl.core": {
            localPaths: ["polyfill-lib/intl/intl.js"],
            meta: {
                localeDir: "node_modules/intl/locale-data/jsonp"
            },
            features: ["internationalization", "javascript.builtins.Intl.PluralRules", "javascript.builtins.Intl.NumberFormat"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_INTL,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "intl.relative-time-format": {
            library: "intl-relative-time-format",
            relativePaths: ["dist/index.js"],
            meta: {
                localeDir: "locale-data"
            },
            features: ["javascript.builtins.Intl.RelativeTimeFormat"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_INTL_RELATIVE_TIME_FORMAT,
            dependencies: ["intl.core", "es.array.includes", "es.object.create", "es.object.is", "es.string.includes", "es.string.replace", "es.symbol.to-string-tag", "es.weak-map"],
            contexts: ALL_CONTEXTS
        },
        "intl.list-format": {
            library: "intl-list-format",
            relativePaths: ["dist/index.js"],
            meta: {
                localeDir: "locale-data"
            },
            features: ["javascript.builtins.Intl.ListFormat"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_INTL_LIST_FORMAT,
            dependencies: ["intl.core", "es.array.includes", "es.object.create", "es.string.replace", "es.symbol.to-string-tag", "es.weak-map"],
            contexts: ALL_CONTEXTS
        },
        animation: {
            polyfills: ["web-animations"]
        },
        "web-animations": {
            library: "web-animations-js",
            relativePaths: ["web-animations.min.js"],
            features: ["web-animation"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_WEB_ANIMATIONS_JS,
            dependencies: ["element", "requestanimationframe"],
            contexts: WINDOW_CONTEXT
        },
        "regenerator-runtime": {
            library: "regenerator-runtime",
            relativePaths: ["runtime.js"],
            features: [],
            version: environment.NPM_PACKAGE_DEPENDENCIES_REGENERATOR_RUNTIME,
            dependencies: ["es.promise"],
            contexts: ALL_CONTEXTS
        },
        template: {
            library: "@webcomponents/template",
            relativePaths: ["template.js"],
            features: ["template"],
            version: environment.NPM_PACKAGE_DEPENDENCIES__WEBCOMPONENTS_TEMPLATE,
            dependencies: ["es"],
            contexts: WINDOW_CONTEXT
        },
        "web-components": {
            polyfills: ["custom-elements", "shadow-dom", "template"]
        },
        "custom-elements": {
            library: "@webcomponents/custom-elements",
            relativePaths: ["custom-elements.min.js"],
            features: ["custom-elementsv1"],
            version: environment.NPM_PACKAGE_DEPENDENCIES__WEBCOMPONENTS_CUSTOM_ELEMENTS,
            dependencies: ["es", "mutation-observer"],
            contexts: WINDOW_CONTEXT
        },
        "shadow-dom": {
            localPaths: [
                "node_modules/@webcomponents/shadydom/shadydom.min.js",
                "node_modules/@webcomponents/shadycss/scoping-shim.min.js",
                "node_modules/@webcomponents/shadycss/custom-style-interface.min.js"
            ],
            features: ["shadowdomv1"],
            version: environment.NPM_PACKAGE_DEPENDENCIES__WEBCOMPONENTS_SHADYDOM,
            dependencies: ["es", "template", "mutation-observer", "event", "node.contains", "queryselector"],
            contexts: WINDOW_CONTEXT
        },
        queryselector: {
            library: "polyfill-service",
            relativePaths: ["polyfills/document.querySelector/polyfill.js"],
            features: ["queryselector"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: ["element", "document", "document-fragment"],
            contexts: WINDOW_CONTEXT
        },
        "document-fragment": {
            // If 'addEventListener' isn't found, the Window interface shouldn't exist on the window
            library: "polyfill-service",
            relativePaths: ["polyfills/DocumentFragment/polyfill.js"],
            features: ["queryselector"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: [],
            contexts: WINDOW_CONTEXT
        },
        "node.parentelement": {
            library: "node.parentelement",
            relativePaths: ["polyfill.min.js"],
            // If 'addEventListener' isn't found, the Window interface shouldn't exist on the window
            features: ["addeventlistener"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_NODE_PARENTELEMENT,
            dependencies: ["document"],
            contexts: WINDOW_CONTEXT
        },
        "scroll-behavior": {
            library: "scroll-behavior-polyfill",
            relativePaths: ["dist/index.js"],
            features: ["css-scroll-behavior", "scrollintoview"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_SCROLL_BEHAVIOR_POLYFILL,
            dependencies: ["es.object.define-property", "es.object.get-own-property-descriptor", "requestanimationframe"],
            contexts: WINDOW_CONTEXT
        },
        "node.contains": {
            library: "polyfill-service",
            relativePaths: ["polyfills/Node/prototype/contains/polyfill.js"],
            // If 'addEventListener' isn't found, the Window interface shouldn't exist on the window
            features: ["addeventlistener"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: ["element"],
            contexts: WINDOW_CONTEXT
        },
        window: {
            library: "polyfill-service",
            relativePaths: ["polyfills/Window/polyfill.js"],
            // If 'addEventListener' isn't found, the Window interface shouldn't exist on the window
            features: ["addeventlistener"],
            version: "1.0.0",
            dependencies: [],
            contexts: WINDOW_CONTEXT
        },
        document: {
            library: "polyfill-service",
            relativePaths: ["polyfills/Document/polyfill.js"],
            // If 'addEventListener' isn't found, the Document interface shouldn't exist on the window
            features: ["addeventlistener"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: [],
            contexts: WINDOW_CONTEXT
        },
        "class-list": {
            library: "polyfill-service",
            relativePaths: ["polyfills/Element/prototype/classList/polyfill.js"],
            features: ["classlist"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: ["dom-token-list"],
            contexts: WINDOW_CONTEXT
        },
        "dom-token-list": {
            library: "polyfill-service",
            relativePaths: ["polyfills/_DOMTokenList/polyfill.js", "polyfills/DOMTokenList/polyfill.js"],
            features: ["rellist"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: ["es.object.define-property"],
            contexts: WINDOW_CONTEXT
        },
        element: {
            library: "polyfill-service",
            relativePaths: ["polyfills/Element/polyfill.js"],
            // If 'addEventListener' isn't found, the Element interface shouldn't exist on the window
            features: ["addeventlistener"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: ["document"],
            contexts: WINDOW_CONTEXT
        },
        event: {
            polyfills: ["event.constructor", "event.focusin", "event.hashchange"]
        },
        "event.constructor": {
            library: "polyfill-service",
            relativePaths: ["polyfills/Event/polyfill.js"],
            features: ["api.Event.Event"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: ["window", "document", "element", "es.object.define-property"],
            contexts: WINDOW_CONTEXT
        },
        "event.focusin": {
            library: "polyfill-service",
            relativePaths: ["polyfills/Event/focusin/polyfill.js"],
            features: ["focusin-focusout-events"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: ["event.constructor"],
            contexts: WINDOW_CONTEXT
        },
        "event.hashchange": {
            library: "polyfill-service",
            relativePaths: ["polyfills/Event/hashchange/polyfill.js"],
            features: ["hashchange"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: ["event.constructor"],
            contexts: WINDOW_CONTEXT
        },
        "custom-event": {
            library: "polyfill-service",
            relativePaths: ["polyfills/CustomEvent/polyfill.js"],
            features: ["customevent"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: ["event"],
            contexts: WINDOW_CONTEXT
        },
        "event-source": {
            library: "polyfill-service",
            relativePaths: ["polyfills/EventSource/polyfill.js"],
            features: ["eventsource"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: [],
            contexts: WINDOW_CONTEXT
        },
        "get-computed-style": {
            library: "polyfill-service",
            relativePaths: ["polyfills/getComputedStyle/polyfill.js"],
            features: ["getcomputedstyle"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_POLYFILL_SERVICE,
            dependencies: ["window"],
            contexts: WINDOW_CONTEXT
        },
        "intersection-observer": {
            library: "intersection-observer",
            relativePaths: ["intersection-observer.js"],
            features: ["intersectionobserver"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_INTERSECTION_OBSERVER,
            dependencies: [
                "get-computed-style",
                "es.array.is-array",
                "es.array.filter",
                "es.array.for-each",
                "es.array.index-of",
                "es.array.map",
                "es.array.some",
                "event",
                "es.function",
                "performance.now"
            ],
            contexts: WINDOW_CONTEXT
        },
        "mutation-observer": {
            library: "mutationobserver-shim",
            relativePaths: ["dist/mutationobserver.min.js"],
            features: ["mutationobserver"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_MUTATIONOBSERVER_SHIM,
            dependencies: [],
            contexts: WINDOW_CONTEXT
        },
        "resize-observer": {
            library: "resize-observer",
            relativePaths: ["dist/resize-observer.min.js"],
            features: ["resizeobserver"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_RESIZE_OBSERVER,
            dependencies: ["get-computed-style", "requestanimationframe"],
            contexts: WINDOW_CONTEXT
        },
        setimmediate: {
            polyfills: ["set-immediate"]
        },
        "set-immediate": {
            library: "setimmediate",
            relativePaths: ["setImmediate.js"],
            features: ["setimmediate"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_SETIMMEDIATE,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        globalthis: {
            polyfills: ["global-this"]
        },
        "global-this": {
            library: "core-js",
            relativePaths: ["modules/esnext.global-this.js"],
            features: ["javascript.builtins.globals.globalThis"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CORE_JS,
            dependencies: [],
            contexts: ALL_CONTEXTS
        },
        "adopted-style-sheets": {
            polyfills: ["constructable-style-sheets"]
        },
        "constructable-style-sheets": {
            library: "construct-style-sheets-polyfill",
            relativePaths: ["adoptedStyleSheets.min.js"],
            features: ["api.DocumentOrShadowRoot.adoptedStyleSheets"],
            version: environment.NPM_PACKAGE_DEPENDENCIES_CONSTRUCT_STYLE_SHEETS_POLYFILL,
            dependencies: ["shadow-dom"],
            contexts: ALL_CONTEXTS
        }
    }
};

// tslint:disable
/**
 * Traces all polyfill names that matches the given name. It may be an alias, and it may refer to additional aliases
 * within the given features
 * @param {PolyfillName} name
 * @returns {Set<PolyfillDealiasedName>}
 */
function traceAllPolyfillNamesForPolyfillName(name) {
    // Get the PolyfillDict that matches the given name
    const match = constant.polyfill[name];
    // If none exists, return an empty array
    if (match == null)
        return new Set();
    // If the PolyfillDict is not an alias, return the name of the polyfill itself
    if (!("polyfills" in match))
        return new Set([name]);
    // Otherwise, recursively trace all polyfill names for each of the polyfill names
    return new Set([].concat.apply([], match.polyfills.map(polyfillName => [...traceAllPolyfillNamesForPolyfillName(polyfillName)])));
}
/**
 * Returns a stringified key as a function of the given polyfill name(s) and encoding
 * @param {IPolyfillFeature | IPolyfillFeatureInput | Set<IPolyfillFeature> | Set<IPolyfillFeatureInput>} name
 * @param {ContentEncodingKind} [encoding]
 * @returns {string}
 */
function getPolyfillIdentifier(name, encoding) {
    const shasum = crypto.createHash("sha1");
    const normalizedName = name instanceof Set ? name : new Set([name]);
    const sortedName = [...normalizedName].sort((a, b) => (a.name < b.name ? -1 : a.name > b.name ? 1 : 0));
    const namePart = sortedName.map(part => `${part.name}${JSON.stringify(part.meta)}${part.context}`).join(",");
    shasum.update(`[${namePart}].${encoding == null ? "" : encoding}`);
    return shasum.digest("hex");
}
/**
 * Returns a stringified key as a function of the given Set of polyfill feature inputs
 * @param {Set<IPolyfillFeatureInput>} polyfills
 * @param {string} userAgent
 * @returns {string}
 */
function getPolyfillSetIdentifier(polyfills, userAgent) {
    const shasum = crypto.createHash("sha1");
    const sortedName = [...polyfills].sort((a, b) => (a.name < b.name ? -1 : a.name > b.name ? 1 : 0));
    const namePart = sortedName.map(part => `${part.name}${JSON.stringify(part.meta)}${JSON.stringify(part.force)}${JSON.stringify(part.context)}`).join(",");
    shasum.update(`[${namePart}].${userAgent}`);
    return shasum.digest("hex");
}
/**
 * Returns a stringified key as a function of the given array of paths to Core-js files
 * @param {string[]} paths
 * @returns {string}
 */
function getCoreJsBundleIdentifier(paths) {
    const shasum = crypto.createHash("sha1");
    const sortedPaths = paths.sort();
    const joinedPaths = sortedPaths.join(",");
    shasum.update(`[${joinedPaths}].corejs`);
    return shasum.digest("hex");
}
/**
 * Returns true if the given polyfill should be included for a particular user agent
 * @param {boolean} force
 * @param {PolyfillContext} context
 * @param {string} userAgent
 * @param {string[]} features
 * @param {Set<PolyfillContext>} supportedContexts
 */
function shouldIncludePolyfill(force, context, userAgent, features, supportedContexts) {
    return supportedContexts.has(context) && (force || features.length < 1 || !browserslistGenerator.userAgentSupportsFeatures(userAgent, ...features));
}
/**
 * Gets the dependors of the given Polyfill name, including those that it must follow if they are - optionally - included
 * @param {PolyfillDealiasedName} polyfillName
 * @param {Set<PolyfillDealiasedName>} includedPolyfillNames
 * @returns {PolyfillDealiasedName[]}
 */
function getEffectiveDependors(polyfillName, includedPolyfillNames) {
    const allOtherPolyfillNames = [...includedPolyfillNames].filter(name => name !== polyfillName);
    const allOtherPolyfills = allOtherPolyfillNames.map(name => [name, constant.polyfill[name]]);
    return allOtherPolyfills
        .filter(([, dict]) => {
        const dependsOnPolyfill = dict.dependencies.some(dependency => traceAllPolyfillNamesForPolyfillName(dependency).has(polyfillName));
        const mustComeAfterThisPolyfill = dict.mustComeAfter != null && (dict.mustComeAfter === "*" || dict.mustComeAfter.some(dependency => traceAllPolyfillNamesForPolyfillName(dependency).has(polyfillName)));
        return dependsOnPolyfill || mustComeAfterThisPolyfill;
    })
        .map(([name]) => name);
}
/**
 * Gets all those polyfills that are required for a user agent
 * @param {Set<IPolyfillFeatureInput>} polyfillSet
 * @param {string} userAgent
 * @returns {[IPolyfillFeature[], Map<PolyfillDealiasedName, number>]}
 */
function getRequiredPolyfillsForUserAgent(polyfillSet, userAgent) {
    const polyfills = [];
    const polyfillNames = new Set();
    const polyfillNameToPolyfillIndexMap = new Map();
    let currentIndex = 0;
    polyfillSet.forEach(polyfill => {
        const match = constant.polyfill[polyfill.name];
        if (shouldIncludePolyfill(polyfill.force, polyfill.context, userAgent, match.features, match.contexts)) {
            const existingIndex = polyfillNameToPolyfillIndexMap.get(polyfill.name);
            if (existingIndex != null) {
                polyfills[existingIndex].meta = polyfill.meta;
                polyfills[existingIndex].context = polyfill.context;
            }
            else {
                polyfills.push({ name: polyfill.name, meta: polyfill.meta, context: polyfill.context });
                polyfillNameToPolyfillIndexMap.set(polyfill.name, currentIndex++);
                polyfillNames.add(polyfill.name);
            }
            const resolvedDependencies = [].concat.apply([], match.dependencies.map(dependency => [...traceAllPolyfillNamesForPolyfillName(dependency)]));
            for (const childPolyfill of getRequiredPolyfillsForUserAgent(new Set(resolvedDependencies.map(dependency => ({ name: dependency, meta: {}, force: polyfill.force, context: polyfill.context }))), userAgent)[0]) {
                if (!polyfillNames.has(childPolyfill.name)) {
                    polyfills.push(Object.assign({}, childPolyfill, { meta: Object.assign({}, childPolyfill.meta, polyfill.meta) }));
                    polyfillNameToPolyfillIndexMap.set(childPolyfill.name, currentIndex++);
                    polyfillNames.add(childPolyfill.name);
                }
            }
        }
    });
    return [polyfills, polyfillNameToPolyfillIndexMap];
}
/**
 * Orders the polyfills given in the Set, including their dependencies
 * @param {Set<IPolyfillFeature>} polyfillSet
 * @param {string} userAgent
 * @returns {Set<IPolyfillFeature>}
 */
async function getOrderedPolyfillsWithDependencies(polyfillSet, userAgent) {
    const [requiredPolyfills, polyfillToIndexMap] = getRequiredPolyfillsForUserAgent(polyfillSet, userAgent);
    const requiredPolyfillNames = new Set(polyfillToIndexMap.keys());
    const requiredPolyfillNamesArray = [...requiredPolyfillNames];
    const graph = [];
    requiredPolyfillNames.forEach(requiredPolyfillName => {
        const dependors = getEffectiveDependors(requiredPolyfillName, requiredPolyfillNames);
        dependors.forEach(dependor => graph.push([requiredPolyfillName, dependor]));
    });
    // Perform topological sorting in a directed acyclic graph
    const result = toposort.array(requiredPolyfillNamesArray, graph);
    // Generate a Set from the results
    return new Set(result.map(name => requiredPolyfills[polyfillToIndexMap.get(name)]));
}
/**
 * Generates an IPolyfillRequest from the given URL
 * @param {URL} url
 * @param {string} userAgent
 * @param {ContentEncodingKind} [encoding]
 * @returns {IPolyfillRequest}
 */
function getPolyfillRequestFromUrl(url, userAgent, encoding) {
    const featuresRaw = url.searchParams.get("features");
    const contextRaw = url.searchParams.get("context");
    let context = contextRaw == null || !POLYFILL_CONTEXTS.includes(contextRaw) ? "window" : contextRaw;
    // Prepare a Set of features
    const featureSet = new Set();
    // If any features are given, walk through them and add to the features set
    if (featuresRaw != null) {
        // Split the comma-separated polyfills
        const splitted = featuresRaw.split(polyfillRawSeparator);
        for (const part of splitted) {
            let force = false;
            const meta = {};
            // The first part will be the polyfill name
            const [name, ...rest] = part.split(polyfillRawDivider);
            // Walk through the rest of the options
            for (const option of rest) {
                const [key, value] = option.split(polyfillOptionKeyValueDivider);
                const splittedValue = value == null ? [] : value.split(polyfillOptionValueSeparator);
                // If the key is 'force', treat it separately
                if (key === polyfillRawForceName) {
                    force = splittedValue.length === 0 || splittedValue.every(booleanize);
                }
                // Otherwise, treat it as meta data
                else {
                    meta[key] = splittedValue.length === 0 ? true : splittedValue.length === 1 ? splittedValue[0] : splittedValue;
                }
            }
            // Trace all polyfill names referenced by the identifier for this polyfill and add all of them to the feature set
            for (const polyfillName of traceAllPolyfillNamesForPolyfillName(name)) {
                // Add it to the set of polyfillable features
                featureSet.add({
                    name: polyfillName,
                    force,
                    meta,
                    context
                });
            }
        }
    }
    // Return the IPolyfillRequest
    return {
        userAgent,
        encoding,
        features: featureSet
    };
}
/**
 * Encodes a PolyfillSet such that it can be embedded in a HTTP header
 * @param {Set<IPolyfillFeature>} polyfillSet
 * @returns {string}
 */
function encodeFeatureSetForHttpHeader(polyfillSet) {
    return stringutil.truncate([...polyfillSet]
        .map(({ meta, name }) => {
        let str = name;
        if (meta != null) {
            const metaEntries = Object.entries(meta);
            if (metaEntries.length > 0) {
                str += " (";
                str += metaEntries.map(([key, value]) => `${key}: ${value}`).join(", ");
                str += `)`;
            }
        }
        return str;
    })
        .join(", "), { length: constant.header.maxChars, omission: "...[omitted]" });
}

/**
 * A Registry of polyfills living in-memory
 */
class MemoryRegistryService {
    constructor() {
        /**
         * A map between keys for registered polyfills and their content
         * @type {Map<string, Buffer>}
         */
        this.registeredPolyfills = new Map();
        /**
         * A map between keys for registered polyfill sets and and their actual sets
         * @type {Map<string, Set<IPolyfillFeature>>}
         */
        this.registeredPolyfillSets = new Map();
        /**
         * A map between keys for registered Core-js bundles and their content
         * @type {Map<string, Buffer>}
         */
        this.registeredCoreJsBundles = new Map();
    }
    /**
     * Gets the contents for the polyfill with the given name and with the given encoding
     * @param {IPolyfillFeature|Set<IPolyfillFeature>} name
     * @param {ContentEncodingKind} [encoding]
     * @returns {Promise<IRegistryGetResult?>}
     */
    async get(name, encoding) {
        const checksum = getPolyfillIdentifier(name, encoding);
        const buffer = this.registeredPolyfills.get(checksum);
        return buffer == null ? undefined : { buffer, checksum };
    }
    /**
     * Gets the Set of Polyfill feature inputs that matches the given input
     * @param {Set<IPolyfillFeatureInput>} input
     * @param {string} userAgent
     */
    async getPolyfillFeatureSet(input, userAgent) {
        const checksum = getPolyfillSetIdentifier(input, userAgent);
        return this.registeredPolyfillSets.get(checksum);
    }
    /**
     * Gets the Core-js bundle that matches the given paths, if any
     * @param {string[]} paths
     * @returns {Promise<Buffer|undefined>}
     */
    async getCoreJsBundle(paths) {
        const checksum = getCoreJsBundleIdentifier(paths);
        return this.registeredCoreJsBundles.get(checksum);
    }
    /**
     * Returns true if a polyfill wil the given name exists
     * @param {IPolyfillFeature|Set<IPolyfillFeature>} name
     * @param {ContentEncodingKind} [encoding]
     * @returns {Promise<boolean>}
     */
    async has(name, encoding) {
        return this.registeredPolyfills.has(getPolyfillIdentifier(name, encoding));
    }
    /**
     * Returns true if a Set of PolyfillFeatures exist in cache for the given PolyfillFeature input Set
     * @param {Set<IPolyfillFeatureInput>} input
     * @param {string} userAgent
     * @returns {Promise<boolean>}
     */
    async hasPolyfillFeatureSet(input, userAgent) {
        return this.registeredPolyfillSets.has(getPolyfillSetIdentifier(input, userAgent));
    }
    /**
     * Returns true if a bundle exists for the given Core-js paths
     * @param {string[]} paths
     * @returns {Promise<boolean>}
     */
    async hasCoreJsBundle(paths) {
        return this.registeredCoreJsBundles.has(getCoreJsBundleIdentifier(paths));
    }
    /**
     * Sets the contents for the polyfill with the given name and of the given encoding
     * @param {IPolyfillFeature|Set<IPolyfillFeature>} name
     * @param {Buffer} buffer
     * @param {ContentEncodingKind} [encoding]
     * @returns {Promise<IRegistryGetResult>}
     */
    async set(name, buffer, encoding) {
        const checksum = getPolyfillIdentifier(name, encoding);
        this.registeredPolyfills.set(checksum, buffer);
        return { buffer, checksum };
    }
    /**
     * Sets the given PolyfillFeature Set for the given Set of PolyfillFeature inputs
     * @param {Set<IPolyfillFeatureInput>} input
     * @param {Set<IPolyfillFeature>} polyfillSet
     * @param {string} userAgent
     * @returns {Promise<Set<IPolyfillFeature>>}
     */
    async setPolyfillFeatureSet(input, polyfillSet, userAgent) {
        const checksum = getPolyfillSetIdentifier(input, userAgent);
        this.registeredPolyfillSets.set(checksum, polyfillSet);
        return polyfillSet;
    }
    /**
     * Sets the given bundle Buffer for the given Core-js paths
     * @param {string[]} paths
     * @param {Buffer} bundle
     */
    async setCoreJsBundle(paths, bundle) {
        const checksum = getCoreJsBundleIdentifier(paths);
        this.registeredCoreJsBundles.set(checksum, bundle);
        return bundle;
    }
}

/**
 * A class that can cache generated Polyfills on disk
 */
class CacheRegistryService {
    constructor(fileSaver, logger, fileLoader, memoryRegistry, config) {
        this.fileSaver = fileSaver;
        this.logger = logger;
        this.fileLoader = fileLoader;
        this.memoryRegistry = memoryRegistry;
        this.config = config;
    }
    /**
     * Initializes the Cache registry
     */
    async initialize() {
        // Flush the entire cache if requested
        if (this.config.clearCache) {
            await this.flushCache();
        }
        // Otherwise, update the disk cache
        await this.updateDiskCache();
    }
    /**
     * Gets the contents for the polyfill with the given name and with the given encoding
     * @param {IPolyfillFeature|Set<IPolyfillFeature>} name
     * @param {ContentEncodingKind} [encoding]
     * @returns {Promise<IRegistryGetResult?>}
     */
    async get(name, encoding) {
        // Attempt to fetch it from the in-memory registry
        const memoryHit = await this.memoryRegistry.get(name, encoding);
        if (memoryHit != null)
            return memoryHit;
        // Otherwise, attempt to get it from cache
        const buffer = await this.getFromCache(this.getCachePath(name, encoding));
        // If not possible, return undefined
        if (buffer == null)
            return undefined;
        // Otherwise, store it in the memory registry and return the Buffer
        return await this.memoryRegistry.set(name, buffer, encoding);
    }
    /**
     * Gets the Set of Polyfill feature inputs that matches the given input
     * @param {Set<IPolyfillFeatureInput>} input
     * @param {string} userAgent
     */
    async getPolyfillFeatureSet(input, userAgent) {
        // Attempt to fetch it from the in-memory registry
        const memoryHit = await this.memoryRegistry.getPolyfillFeatureSet(input, userAgent);
        if (memoryHit != null)
            return memoryHit;
        // Otherwise, attempt to get it from cache
        const buffer = await this.getFromCache(this.getCachePathForPolyfillSet(input, userAgent));
        // If not possible, return undefined
        if (buffer == null)
            return undefined;
        // Otherwise, store it in the memory registry and return the Buffer
        return await this.memoryRegistry.setPolyfillFeatureSet(input, new Set(JSON.parse(buffer.toString())), userAgent);
    }
    /**
     * Gets the Core-js bundle that matches the given paths
     * @param {string[]} paths
     * @returns {Promise<Buffer|undefined>}
     */
    async getCoreJsBundle(paths) {
        // Attempt to fetch it from the in-memory registry
        const memoryHit = await this.memoryRegistry.getCoreJsBundle(paths);
        if (memoryHit != null)
            return memoryHit;
        // Otherwise, attempt to get it from cache
        const buffer = await this.getFromCache(this.getCachePathForCoreJsBundle(paths));
        // If not possible, return undefined
        if (buffer == null)
            return undefined;
        // Otherwise, store it in the memory registry and return the Buffer
        return await this.memoryRegistry.setCoreJsBundle(paths, buffer);
    }
    /**
     * Returns true if a polyfill wil the given name exists
     * @param {IPolyfillFeature|Set<IPolyfillFeature>} name
     * @param {ContentEncodingKind} [encoding]
     * @returns {Promise<boolean>}
     */
    async has(name, encoding) {
        return (await this.get(name, encoding)) != null;
    }
    /**
     * Returns true if there is a PolyfillFeature Set in the cache that matches the given input Set
     * @param {Set<IPolyfillFeatureInput>} input
     * @param {string} userAgent
     */
    async hasPolyfillFeatureSet(input, userAgent) {
        return (await this.getPolyfillFeatureSet(input, userAgent)) != null;
    }
    /**
     * Returns true if there is a Core-js bundle in cache that matches the given paths
     * @param {string[]} paths
     * @returns {Promise<boolean>}
     */
    async hasCoreJsBundle(paths) {
        return (await this.getCoreJsBundle(paths)) != null;
    }
    /**
     * Sets the contents for the polyfill with the given name and of the given encoding
     * @param {IPolyfillFeature|Set<IPolyfillFeature>} name
     * @param {Buffer} contents
     * @param {ContentEncodingKind} [encoding]
     * @returns {Promise<IRegistryGetResult>}
     */
    async set(name, contents, encoding) {
        // Add it to the memory cache as well as the disk cache
        await this.writeToCache(this.getCachePath(name, encoding), contents);
        return await this.memoryRegistry.set(name, contents, encoding);
    }
    /**
     * Sets the given PolyfillFeature Set for the given Set of PolyfillFeature inputs
     * @param {Set<IPolyfillFeatureInput>} input
     * @param {Set<IPolyfillFeature>} polyfillSet
     * @param {string} userAgent
     * @returns {Promise<Set<IPolyfillFeature>>}
     */
    async setPolyfillFeatureSet(input, polyfillSet, userAgent) {
        // Add it to the memory cache as well as the disk cache
        await this.writeToCache(this.getCachePathForPolyfillSet(input, userAgent), Buffer.from(JSON.stringify([...polyfillSet])));
        return await this.memoryRegistry.setPolyfillFeatureSet(input, polyfillSet, userAgent);
    }
    /**
     * Sets the Core-js bundle that matches the given paths in the cache
     * @param {string[]} paths
     * @param {Buffer} bundle
     */
    async setCoreJsBundle(paths, bundle) {
        // Add it to the memory cache as well as the disk cache
        await this.writeToCache(this.getCachePathForCoreJsBundle(paths), bundle);
        return await this.memoryRegistry.setCoreJsBundle(paths, bundle);
    }
    /**
     * Returns true if the given polyfill name needs an update within the cache
     * @param {PolyfillName} polyfillName
     * @param {string} currentVersion
     * @returns {Promise<boolean>}
     */
    async needsUpdate(polyfillName, currentVersion) {
        const packageVersionMap = await this.getPackageVersionMap();
        const cachedVersion = packageVersionMap[polyfillName];
        return cachedVersion == null || isNaN(parseInt(cachedVersion)) || semver.gt(semver.coerce(currentVersion).version, semver.coerce(cachedVersion).version);
    }
    /**
     * Updates the cached package version map
     * @param {IterableIterator<PolyfillName, string>} options
     * @returns {Promise<void>}
     */
    async updatePackageVersionMap(options) {
        const packageVersionMap = await this.getPackageVersionMap();
        for (const [name, version] of options) {
            packageVersionMap[name] = version;
        }
        await this.fileSaver.save(constant.path.cachePackageVersionMap, JSON.stringify(packageVersionMap, null, " "));
    }
    /**
     * Updates the disk cache. If the cache is already valid, there is nothing to do.
     * Otherwise, it will store all polyfilled libraries and their current versions
     * @returns {Promise<void>}
     */
    async updateDiskCache() {
        // Start with validating the disk cache
        const cacheIsValid = await this.validateDiskCache();
        // If the cache is valid, do no more
        if (cacheIsValid)
            return;
        // Otherwise, build a map of libraries and their versions
        const libraryToVersionMap = new Map();
        await Promise.all(Object.entries(constant.polyfill).map(async ([polyfillName, polyfill]) => {
            // Skip aliases
            if ("polyfills" in polyfill)
                return;
            // Map the version to the library name
            if (!libraryToVersionMap.has(polyfillName)) {
                libraryToVersionMap.set(polyfillName, polyfill.version);
            }
        }));
        // Update the disk cache
        await this.updatePackageVersionMap(libraryToVersionMap.entries());
    }
    /**
     * Validates the disk cache. If any version of any package has been updated,
     * the entire cache will be flushed.
     * If 'true' is returned, the cache is valid
     * @returns {Promise<boolean>}
     */
    async validateDiskCache() {
        for (const [polyfillName, polyfill] of Object.entries(constant.polyfill)) {
            // Skip aliases
            if ("polyfills" in polyfill)
                continue;
            // If the local copy of the polyfill needs to be updated, flush the entire cache
            if (await this.needsUpdate(polyfillName, polyfill.version)) {
                this.logger.debug(`${polyfillName} needs an update! Flushing cache...`);
                await this.flushCache();
                return false;
            }
        }
        return true;
    }
    /**
     * Flushes the cache entirely
     * @returns {Promise<void>>}
     */
    async flushCache() {
        await this.fileSaver.remove(constant.path.cacheRoot);
    }
    /**
     * Writes the given Buffer to cache
     * @param {string} path
     * @param {Buffer} content
     * @returns {Promise<void>}
     */
    async writeToCache(path, content) {
        await this.fileSaver.save(path, content);
    }
    /**
     * Gets the package version map from the cache. A new one will be created if it doesn't exist already
     * @returns {object}
     */
    async getPackageVersionMap() {
        let packageVersionMap;
        const packageVersionMapRaw = await this.getFromCache(constant.path.cachePackageVersionMap);
        packageVersionMap = packageVersionMapRaw == null ? {} : JSON.parse(packageVersionMapRaw.toString());
        return packageVersionMap;
    }
    /**
     * Returns the contents on the given path from the cache
     * @param {string} path
     * @returns {Promise<void>}
     */
    async getFromCache(path) {
        try {
            return await this.fileLoader.load(path);
        }
        catch (_a) {
            return undefined;
        }
    }
    /**
     * Gets the cache path to the given name and encoding
     * @param {IPolyfillFeature | Set<IPolyfillFeature>} name
     * @param {ContentEncodingKind} encoding
     * @returns {string}
     */
    getCachePath(name, encoding) {
        return path.join(constant.path.cacheRoot, getPolyfillIdentifier(name, encoding));
    }
    /**
     * Gets the cache path to the given Polyfill Feature Set
     * @param {Set<IPolyfillFeatureInput>} input
     * @param {string} userAgent
     * @returns {string}
     */
    getCachePathForPolyfillSet(input, userAgent) {
        return path.join(constant.path.cacheRoot, getPolyfillSetIdentifier(input, userAgent));
    }
    /**
     * Gets the cache path to the bundle generated for the given Core-js paths
     * @param {string[]} paths
     * @returns {string}
     */
    getCachePathForCoreJsBundle(paths) {
        return path.join(constant.path.cacheRoot, getCoreJsBundleIdentifier(paths));
    }
    static get [Symbol.for("___CTOR_ARGS___")]() { return ["IFileSaver", "ILoggerService", "IFileLoader", "IMemoryRegistryService", "IConfig"]; }
}

/**
 * Ensures that the given item is in fact an array
 * @param {T[] | T} item
 * @returns {T[]}
 */
function ensureArray(item) {
    return Array.isArray(item) ? item : [item];
}

const SYNC_OPTIONS = { cwd: __dirname };
/**
 * A service that can load and cache all polyfills
 */
class PolyfillBuilderService {
    constructor(minifier, flattener, logger, cacheService, compressor, fileLoader) {
        this.minifier = minifier;
        this.flattener = flattener;
        this.logger = logger;
        this.cacheService = cacheService;
        this.compressor = compressor;
        this.fileLoader = fileLoader;
    }
    /**
     * Builds the given PolyfillSet and returns the result in all encodings
     * @param {Set<IPolyfillFeature>} polyfillSet
     * @returns {Promise<ICompressedPolyfillSetResult>}
     */
    async buildPolyfillSet(polyfillSet) {
        const input = [];
        let content = "";
        let hasAddedCoreJsContent = false;
        // Take all Core Js paths
        const coreJsPaths = [].concat.apply([], [...polyfillSet].map(polyfillFeature => {
            if (!this.isCoreJs(polyfillFeature))
                return [];
            const match = constant.polyfill[polyfillFeature.name];
            const relativePaths = Array.isArray(match.relativePaths) ? match.relativePaths : match.relativePaths[polyfillFeature.context];
            return relativePaths.map(relativePath => relativePath.replace("modules/", "").replace(".js", ""));
        }));
        // Check if a bundle has been built previously for the given core js paths
        let coreJsContent = await this.cacheService.getCoreJsBundle(coreJsPaths);
        // If not, build a bundle and store within the cache
        if (coreJsContent == null) {
            // Build a bundle of the CoreJs paths
            coreJsContent = buffer.Buffer.from(coreJsPaths.length < 1 ? "" : await builder({ modules: coreJsPaths }));
            // Store it within the cache
            await this.cacheService.setCoreJsBundle(coreJsPaths, coreJsContent);
        }
        else {
            this.logger.debug(`Matched CoreJS paths in cache!`);
        }
        for (const polyfillFeature of polyfillSet) {
            // If this polyfill feature represents Core Js, add its' content to the bundle unless it has been added previously
            if (this.isCoreJs(polyfillFeature)) {
                if (!hasAddedCoreJsContent) {
                    hasAddedCoreJsContent = true;
                    content += `\n${coreJsContent.toString()}`;
                }
                continue;
            }
            const absolutePaths = [];
            const match = constant.polyfill[polyfillFeature.name];
            if (match == null || "polyfills" in match) {
                throw new TypeError(`No aliased polyfill names can be built! These must be resolved before calling ${this.buildPolyfillSet.name}!`);
            }
            const flatten = match.flatten === true;
            const rootDirectory = "library" in match ? path.join("node_modules", typeof match.library === "string" ? match.library : match.library[polyfillFeature.context]) : "";
            const localPaths = "library" in match
                ? Array.isArray(match.relativePaths)
                    ? match.relativePaths
                    : match.relativePaths[polyfillFeature.context]
                : Array.isArray(match.localPaths)
                    ? match.localPaths
                    : match.localPaths[polyfillFeature.context];
            const { meta } = match;
            // If SystemJS is requested, the variant to use may be provided as metadata. If so, we should use that one, rather than the relativePaths
            if (polyfillFeature.name === "systemjs" &&
                meta != null &&
                polyfillFeature.meta != null &&
                "variant" in polyfillFeature.meta &&
                (polyfillFeature.meta.variant === "s" || polyfillFeature.meta.variant === "system")) {
                const metaVariantPathInput = path.join(rootDirectory, meta[polyfillFeature.meta.variant]);
                const resolvedMetaVariantPath = findUp.sync(metaVariantPathInput, SYNC_OPTIONS);
                if (resolvedMetaVariantPath != null) {
                    absolutePaths.push(resolvedMetaVariantPath);
                }
                else {
                    this.logger.debug(`Unresolved path:`, metaVariantPathInput);
                }
            }
            // Otherwise, use all of the given relativePaths
            else {
                // For each of the relative paths, compute the absolute path
                for (const path$1 of localPaths) {
                    const pathInput = path.join(rootDirectory, path$1);
                    const resolvedPath = findUp.sync(pathInput, SYNC_OPTIONS);
                    if (resolvedPath != null) {
                        absolutePaths.push(resolvedPath);
                    }
                    else {
                        this.logger.debug(`Unresolved path:`, pathInput);
                    }
                }
            }
            // If Zone is requested, 'zone-error' may be requested which improves the produced Stack trace when using Zone
            if (polyfillFeature.name === "zone" && meta != null && polyfillFeature.meta != null && polyfillFeature.meta.error === true) {
                const errorPathInput = path.join(rootDirectory, meta.error);
                const resolvedErrorPath = findUp.sync(errorPathInput, SYNC_OPTIONS);
                if (resolvedErrorPath != null) {
                    absolutePaths.push(resolvedErrorPath);
                }
                else {
                    this.logger.debug(`Unresolved path:`, errorPathInput);
                }
            }
            // If the Polyfill is "intl.core" and a localeDir is associated with it, also resolve the requested locales (if any)
            if (polyfillFeature.name.startsWith("intl.") && meta != null && "localeDir" in meta && polyfillFeature.meta.locale != null) {
                // Normalize the requested locales to make sure we have an array to work with
                const requestedLocales = ensureArray(polyfillFeature.meta.locale);
                // Loop through all of the requested locales in parallel
                await Promise.all(requestedLocales.map(async (requestedLocale) => {
                    // Resolve the absolute path
                    const localePathInput = path.join(rootDirectory, meta.localeDir, `${requestedLocale}.js`);
                    const resolvedLocalePath = findUp.sync(localePathInput, SYNC_OPTIONS);
                    if (resolvedLocalePath != null) {
                        absolutePaths.push(resolvedLocalePath);
                    }
                    else {
                        this.logger.debug(`Unresolved path:`, localePathInput);
                    }
                }));
            }
            // Push all of the absolute paths for this specific polyfill to the input paths
            input.push({
                polyfillName: polyfillFeature.name,
                paths: absolutePaths,
                flatten
            });
        }
        // Load all of the input paths and add them to the generated content
        for (const { paths, flatten } of input) {
            if (flatten) {
                content += `\n${await this.flattener.flatten({
                    path: paths
                })}`;
            }
            else {
                for (const path of paths) {
                    content += `\n${(await this.fileLoader.load(path)).toString()}`;
                }
            }
        }
        // Minify the result
        const minified = buffer.Buffer.from(await this.minifier.minify({ code: buffer.Buffer.from(content) }));
        // Compress the result
        const { brotli, zlib } = await this.compressor.compress(minified);
        // Return all of the Buffers
        return {
            brotli,
            minified,
            zlib
        };
    }
    /**
     * Returns true if the given polyfill feature is core-js
     * @param {IPolyfillFeature} polyfillFeature
     * @returns {boolean}
     */
    isCoreJs(polyfillFeature) {
        const match = constant.polyfill[polyfillFeature.name];
        return "library" in match && match.library === "core-js";
    }
    static get [Symbol.for("___CTOR_ARGS___")]() { return ["IMinifyService", "IFlattenerService", "ILoggerService", "ICacheRegistryService", "ICompressorService", "IFileLoader"]; }
}



/**
 * Gets certificates that can be used with a HTTP2 server
 * @param {IGetCertificateOptions} options
 * @returns {Promise<{key: Buffer|undefined, cert: Buffer|undefined}>}
 */
async function getCertificates({ key, cert, production, host, https }) {
    // If a key and certificate was already given, use those two
    if (key != null && cert != null)
        return { key, cert };
    // Otherwise, if not in production, use devcert to generate a new certificate
    if (!production && https) {
        return await devcert.certificateFor(host);
    }
    // Return nothing
    return { key: undefined, cert: undefined };
}

/**
 * Helps with working with the API
 */
class ApiService {
    constructor(server, config) {
        this.server = server;
        this.config = config;
    }
    /**
     * Launches the API.
     * @returns {Promise<void>}
     */
    async launch() {
        // Serve if necessary
        if (this.server.initializing || this.server.hasInitialized) {
            await this.server.onInitialized();
        }
        else {
            await this.server.serve(Object.assign({ http2: this.config.http2, host: this.config.host, port: this.config.port, root: process.cwd() }, (await getCertificates(this.config))));
        }
    }
    static get [Symbol.for("___CTOR_ARGS___")]() { return ["IServer", "IConfig"]; }
}



// tslint:disable:no-any
// tslint:disable:no-duplicate-string
const GET_COLOR = chalk.green;
const PUT_COLOR = chalk.yellow;
const DELETE_COLOR = chalk.red;
const POST_COLOR = chalk.blue;
const OPTIONS_COLOR = chalk.gray;
/**
 * Splits a header that represents a comma-separated list
 * @param {string} header
 * @returns {Set<string>}
 */
function splitStringifiedListHeader(header) {
    const splitted = Array.isArray(header) ? header : header.split(",").map(part => part.trim());
    return new Set(splitted);
}
/**
 * Gets an IRequest from the given IncomingHttpHeaders
 * @param {IncomingHttpHeaders} headers
 * @param {boolean} http2
 * @returns {Request}
 */
function getRequestFromIncomingHeaders(headers, http2) {
    let path = headers[":path"];
    if (typeof path === "string") {
        // Replace all literal "+" with its literal encoded variant
        path = path.replace(/\+/g, "%2B");
    }
    return {
        http2,
        method: headers[":method"],
        accept: headers.accept == null ? undefined : splitStringifiedListHeader(headers.accept),
        acceptEncoding: headers["accept-encoding"] == null ? undefined : splitStringifiedListHeader(headers["accept-encoding"]),
        acceptLanguage: headers["accept-language"] == null ? undefined : splitStringifiedListHeader(headers["accept-language"]),
        userAgent: headers["user-agent"],
        // @ts-ignore
        url: new url.URL(path, `${headers[":scheme"]}://${headers[":authority"]}`),
        cachedChecksum: headers["if-none-match"]
    };
}
/**
 * Paints the given method with Chalk. Each method has its' own color
 * @param {string} method
 * @returns {string}
 */
function paintMethod(method) {
    switch (method) {
        case "GET":
            return GET_COLOR(method);
        case "PUT":
            return PUT_COLOR(method);
        case "POST":
            return POST_COLOR(method);
        case "DELETE":
            return DELETE_COLOR(method);
        case "OPTIONS":
            return OPTIONS_COLOR(method);
    }
}
/**
 * Prints the given request
 * @param {IRequest} request
 */
function printRequest({ method, url, userAgent }) {
    console.log(`${paintMethod(method)} ${url.toString()} (${chalk.gray(userAgent)})`);
}
/**
 * Picks the most favorable encoding to use from the given Set of encodings
 * @param {Set<string> | undefined} encodings
 * @returns {ContentEncodingKind | undefined}
 */
function pickEncoding(encodings) {
    if (encodings == null)
        return undefined;
    if (encodings.has(ContentEncodingKind.BROTLI))
        return ContentEncodingKind.BROTLI;
    if (encodings.has(ContentEncodingKind.GZIP))
        return ContentEncodingKind.GZIP;
    return undefined;
}

// tslint:disable:no-any
/**
 * Responds to the given request with the given response
 * @param {HttpServerResponse} rawResponse
 * @param {Response} response
 * @returns {void}
 */
function respondToRequest(rawResponse, response) {
    if ("contentType" in response) {
        rawResponse.setHeader("Content-Type", response.contentType);
    }
    if ("body" in response) {
        rawResponse.setHeader("Content-Length", Buffer.byteLength(response.body));
    }
    if ("cacheControl" in response && response.cacheControl != null) {
        rawResponse.setHeader("Cache-Control", response.cacheControl);
    }
    if ("contentEncoding" in response && response.contentEncoding != null) {
        rawResponse.setHeader("Content-Encoding", response.contentEncoding);
    }
    if ("checksum" in response && response.checksum != null) {
        rawResponse.setHeader("ETag", response.checksum);
    }
    if ("polyfillsHeader" in response && response.polyfillsHeader != null) {
        rawResponse.setHeader(constant.header.polyfills, response.polyfillsHeader);
    }
    // Allow any origin
    rawResponse.setHeader("Access-Control-Allow-Origin", "*");
    // Set the status code
    rawResponse.statusCode = response.statusCode;
    // Submit the response
    rawResponse.end("body" in response ? response.body : undefined);
}

// tslint:disable:no-duplicate-string
/**
 * A HTTP2 Server
 */
class Server {
    constructor(logger, requestHandler) {
        this.logger = logger;
        this.requestHandler = requestHandler;
        /**
         * Whether or not a server is currently being initialized
         * @type {boolean}
         */
        this.initializing = false;
        /**
         * Whether or not the server has initialized
         * @type {boolean}
         */
        this.hasInitialized = false;
        /**
         * The constructed Server
         * @type {Http2SecureServer|Http2Server|null}
         */
        this.server = null;
        /**
         * All callbacks that will be invoked when the server has initialized
         * @type {Set<OnInitializedCallback>}
         */
        this.onInitializedCallbacks = new Set();
    }
    /**
     * Starts a new HTTP2 server and listens on the given port
     * @param {IServeOptions} options
     * @returns {Promise<void>}
     */
    async serve(options) {
        if (this.initializing || this.hasInitialized)
            return;
        if (this.server != null)
            await this.stop();
        this.initializing = true;
        this.hasInitialized = false;
        // Only run a TLS-encrypted server if a key and cert is given
        const shouldRunSecureServer = options.key != null && options.cert != null;
        const http2RequestHandler = async (request, response, tls) => this.onRequest(request, response, options, tls);
        if (options.http2) {
            // Create HTTP2 server
            this.server = shouldRunSecureServer
                ? http2.createSecureServer({ key: options.key, cert: options.cert, allowHTTP1: true }, async (request, response) => http2RequestHandler(request, response, true))
                : http2.createServer({ allowHTTP1: true }, async (request, response) => http2RequestHandler(request, response, false));
        }
        else {
            // Create HTTP2 server
            this.server = shouldRunSecureServer
                ? https.createServer({ key: options.key, cert: options.cert }, async (request, response) => http2RequestHandler(request, response, true))
                : http.createServer(async (request, response) => http2RequestHandler(request, response, false));
        }
        // Start listening on the given port
        await new Promise(resolve => this.server.listen({
            port: options.port,
            path: options.root
        }, () => {
            const startMessage = options.http2 ? `${shouldRunSecureServer ? "Secure " : ""}HTTP2 ` : `${shouldRunSecureServer ? "HTTPS " : "HTTP "}`;
            const endMessage = `Server is listening on ${options.host}:${options.port}`;
            this.logger.log(`${startMessage}${endMessage}`);
            this.initializing = false;
            this.hasInitialized = true;
            this.onInitializedCallbacks.forEach(callback => callback());
            this.onInitializedCallbacks.clear();
            resolve();
        }));
    }
    /**
     * Stops the active server, if any
     * @returns {Promise<void>}
     */
    async stop() {
        return new Promise(resolve => {
            this.hasInitialized = false;
            // If there is not server to stop, do nothing
            if (this.server == null)
                return resolve();
            this.server.close(() => {
                resolve();
            });
            this.server = null;
        });
    }
    /**
     * Adds a callback to the Set of callbacks to invoke when polyfills has been successfully built
     * @returns {Promise<void>}
     */
    async onInitialized() {
        return new Promise(resolve => {
            if (!this.initializing && this.hasInitialized)
                return resolve();
            else
                this.onInitializedCallbacks.add(resolve);
        });
    }
    /**
     * Listens for "stream" events on the given Http2SecureServer
     * @param {HttpIncomingMessage|Http2ServerRequest} request
     * @param {HttpServerResponse|Http2ServerResponse} response
     * @param {IServeOptions} serverOptions
     * @param {boolean} tls
     * @returns {Promise<void>}
     */
    async onRequest(request, response, serverOptions, tls) {
        const isHttp2 = "stream" in request;
        // Format a request
        const normalizedRequest = getRequestFromIncomingHeaders(Object.assign({}, request.headers, { ":method": ":method" in request.headers ? request.headers[":method"] : request.method, ":path": ":path" in request.headers ? request.headers[":path"] : request.url, ":scheme": ":scheme" in request.headers ? request.headers[":scheme"] : tls ? "https" : "http", ":authority": ":authority" in request.headers ? request.headers[":authority"] : request.headers.host }), isHttp2);
        const handleOptions = {
            request: normalizedRequest,
            serverOptions
        };
        // Respond with the result of calling the handler
        respondToRequest(response, await this.requestHandler.handle(handleOptions));
    }
    static get [Symbol.for("___CTOR_ARGS___")]() { return ["ILoggerService", "IRequestHandler"]; }
}

/**
 * A handler that can handle requests
 */
class RequestHandler {
    constructor(controllers, staticController) {
        this.controllers = controllers;
        this.staticController = staticController;
    }
    /**
     * Handles the given request
     * @param {RequestHandlerOptions} options
     * @returns {Promise<Response>}
     */
    async handle(options) {
        // Print the request
        printRequest(options.request);
        switch (options.request.method) {
            case "GET":
                return await this.handleGetRequest(options);
            case "OPTIONS":
                return await this.handleOptionsRequest(options);
            default:
                return {
                    body: `Only GET requests are supported`,
                    statusCode: options.request.http2 ? http2.constants.HTTP_STATUS_METHOD_NOT_ALLOWED : httpStatusCodes.METHOD_NOT_ALLOWED,
                    contentType: "text/plain"
                };
        }
    }
    /**
     * Handles the given GET request
     * @param {RequestHandlerOptions} options
     * @returns {Promise<Response>}
     */
    async handleGetRequest(options) {
        // Find the first matched controller
        for (const controller of this.controllers) {
            const match = controller.match(options.request);
            if (match != null) {
                return await match(options.request);
            }
        }
        // Fall back to the index route
        return await this.staticController.onIndexRequested(options.request);
    }
    /**
     * Handles the given OPTIONS request
     * @param {RequestHandlerOptions} options
     * @returns {Promise<Response>}
     */
    async handleOptionsRequest(options) {
        return {
            statusCode: options.request.http2 ? http2.constants.HTTP_STATUS_OK : httpStatusCodes.OK
        };
    }
    static get [Symbol.for("___CTOR_ARGS___")]() { return ["RegisteredControllers", "IStaticController"]; }
}

/**
 * Generates some HTML contents with the given message
 * @param {string} message
 * @returns {string}
 */
function generateHtml(message) {
    // language=HTML
    return `
		<!DOCTYPE html>
		<html lang="en">
		<head>
			<meta charset="UTF-8">
			<title>${constant.meta.name} v${constant.meta.version}</title>
			<style>
				body {
					background: rgba(0,0,0,.1);
					font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";
				}
			</style>
		</head>
		<body>
			${message}
		</body>
		</html>
	`;
}
/**
 * Generates some HTML contents with the given message
 * @param {string} message
 * @param {number} code
 * @returns {string}
 */
function generateErrorHtml(message, code) {
    // language=HTML
    return `
		<!DOCTYPE html>
		<html lang="en">
		<head>
			<meta charset="UTF-8">
			<title>${constant.meta.name} v${constant.meta.version}</title>
		</head>
		<body>
			<h1>An Error occurred.</h1>
			<h5>Code: ${code}</h5>
			<p>${message.name}</p>
			<p>${message.stack}</p>
			${message}
		</body>
		</html>
	`;
}

/**
 * Business logic for static files
 */
class StaticBl {
    /**
     * Generates a HTML-formatted welcome message
     * @returns {Promise<string>}
     */
    async getWelcomeMessage() {
        return generateHtml(`
			<div style="display: flex; flex-direction: row;">
				<img style="margin-right: 30px" alt="logo" height="100" src="https://raw.githubusercontent.com/wessberg/Polyfiller/master/documentation/asset/logo-color.png" />
				<h1 style="box-shadow: 0 1px 0 0 rgba(0,0,0,1); display: inline-block">Welcome to ${constant.meta.name} v${constant.meta.version}</h1>
			</div>
			<h3>To use it, please send a request to the path: <code>${constant.endpoint.polyfill}</code> instead.</h3>
			<h3>For API reference, please see <a href="${constant.meta.github}">Github</a></h3>
			`);
    }
}

/**
 * Business logic for polyfills
 */
class PolyfillBl {
    constructor(cacheRegistry, logger, builder) {
        this.cacheRegistry = cacheRegistry;
        this.logger = logger;
        this.builder = builder;
    }
    /**
     * Generates a chunk of polyfills that matches the given request
     * @param {IPolyfillRequest} request
     * @returns {Promise<IGetPolyfillsResult>}
     */
    async getPolyfills(request) {
        // Check if a polyfill set exists within the cache for the request features and the user agent of the request
        let featureSet = await this.cacheRegistry.getPolyfillFeatureSet(request.features, request.userAgent);
        // If not, resolve and order the required polyfills
        if (featureSet == null) {
            featureSet = await getOrderedPolyfillsWithDependencies(new Set([...request.features]
                // Take only valid names
                .filter(feature => feature.name in constant.polyfill)), request.userAgent);
            // Store them within the cache
            await this.cacheRegistry.setPolyfillFeatureSet(request.features, featureSet, request.userAgent);
        }
        else {
            this.logger.debug("Matched Polyfill Set in cache!");
        }
        this.logger.debug(featureSet);
        // Check if a Set has already been registered for this combination
        const existingSet = await this.cacheRegistry.get(featureSet, request.encoding);
        if (existingSet != null) {
            this.logger.debug("Matched Polyfills in cache!");
            // If it has, just return that one
            return { result: existingSet, featureSet };
        }
        // Otherwise, build the polyfills and store them in the Cache Registry before returning them
        const { brotli, minified, zlib } = await this.builder.buildPolyfillSet(featureSet);
        // Add the polyfills to the registry
        const [minifiedResult, brotliResult, zlibResult] = await Promise.all([
            this.cacheRegistry.set(featureSet, minified),
            this.cacheRegistry.set(featureSet, brotli, ContentEncodingKind.BROTLI),
            this.cacheRegistry.set(featureSet, zlib, ContentEncodingKind.GZIP)
        ]);
        // Return the joined Buffer
        switch (request.encoding) {
            case ContentEncodingKind.BROTLI:
                return { result: brotliResult, featureSet };
            case ContentEncodingKind.GZIP:
                return { result: zlibResult, featureSet };
            case undefined:
                return { result: minifiedResult, featureSet };
        }
    }
    static get [Symbol.for("___CTOR_ARGS___")]() { return ["ICacheRegistryService", "ILoggerService", "IPolyfillBuilderService"]; }
}

// tslint:disable:no-any
/**
 * A controller can handle a request and return a response for it
 */
class Controller {
    constructor() {
        // noinspection JSPotentiallyInvalidConstructorUsage
        /**
         * The controller method listeners of this concrete Controller instance
         * @type {Map<RegExp, ControllerMethod>}
         */
        this.controllerMethods = this.constructor.prototype.controllerMethods;
    }
    /**
     * Returns a ControllerMatch for the given request, if any exists
     * @param {Request} request
     * @returns {ControllerMethod | undefined}
     */
    match(request) {
        const controllerMethodListener = this.controllerMethods.get(request.method);
        if (controllerMethodListener == null) {
            return undefined;
        }
        // Find the listener that matches the request
        for (const [path, method] of controllerMethodListener.entries()) {
            const normalizedPath = Array.isArray(path) ? path : [path];
            for (const part of normalizedPath) {
                if (typeof part === "string" ? part === request.url.pathname : part.test(request.url.pathname)) {
                    return method.bind(this);
                }
            }
        }
        return undefined;
    }
}



// tslint:disable:no-any
/**
 * A decorator that registers a method as a GET handler
 * @param {RegExp} path
 * @returns {(target: Object) => any}
 * @constructor
 */
function GET({ path }) {
    return function (target, name) {
        if (target.controllerMethods == null)
            target.controllerMethods = new Map();
        let baseMap = target.controllerMethods.get("GET");
        const hadBaseMap = baseMap != null;
        if (!hadBaseMap) {
            baseMap = new Map();
        }
        baseMap.set(path, target[name]);
        if (!hadBaseMap) {
            target.controllerMethods.set("GET", baseMap);
        }
    };
}

/**
 * A controller that can respond to requests for static resources
 */
class StaticController extends Controller {
    constructor(staticBl) {
        super();
        this.staticBl = staticBl;
    }
    /**
     * Called when the index (/) is requested
     * @param {Request} request
     * @returns {Promise<Response>}
     */
    async onIndexRequested(request) {
        // Return an OK
        return {
            contentType: "text/html",
            statusCode: request.http2 ? http2.constants.HTTP_STATUS_OK : httpStatusCodes.OK,
            body: await this.staticBl.getWelcomeMessage()
        };
    }
    static get [Symbol.for("___CTOR_ARGS___")]() { return ["IStaticBl"]; }
}
tslib_1.__decorate([
    GET({ path: constant.endpoint.index })
], StaticController.prototype, "onIndexRequested", null);

/**
 * A controller that can respond to requests for polyfills
 */
class PolyfillController extends Controller {
    constructor(polyfillBl) {
        super();
        this.polyfillBl = polyfillBl;
    }
    /**
     * Called when the polyfill (/polyfill) endpoint is requested
     * @param {Request} request
     * @returns {Promise<Response>}
     */
    async onPolyfillRequested(request) {
        // Normalize the polyfill request
        const polyfillRequest = getPolyfillRequestFromUrl(request.url, request.userAgent, pickEncoding(request.acceptEncoding));
        // Generate polyfill bundle
        try {
            const { result: { buffer, checksum }, featureSet } = await this.polyfillBl.getPolyfills(polyfillRequest);
            // If the cached checksum (ETag) is identical, respond with NOT_MODIFIED
            if (request.cachedChecksum != null && request.cachedChecksum === checksum) {
                return {
                    statusCode: request.http2 ? http2.constants.HTTP_STATUS_NOT_MODIFIED : httpStatusCodes.NOT_MODIFIED,
                    cacheControl: "public,max-age=31536000,immutable",
                    polyfillsHeader: encodeFeatureSetForHttpHeader(featureSet)
                };
            }
            // Return an OK
            return {
                contentType: "application/javascript",
                statusCode: request.http2 ? http2.constants.HTTP_STATUS_OK : httpStatusCodes.OK,
                body: buffer,
                cacheControl: "public,max-age=31536000,immutable",
                contentEncoding: polyfillRequest.encoding,
                checksum,
                polyfillsHeader: encodeFeatureSetForHttpHeader(featureSet)
            };
        }
        catch (ex) {
            // Respond with error code 500
            const statusCode = request.http2 ? http2.constants.HTTP_STATUS_INTERNAL_SERVER_ERROR : httpStatusCodes.INTERNAL_SERVER_ERROR;
            return {
                contentType: "text/html",
                statusCode,
                body: generateErrorHtml(ex, statusCode)
            };
        }
    }
    static get [Symbol.for("___CTOR_ARGS___")]() { return ["IPolyfillBl"]; }
}
tslib_1.__decorate([
    GET({ path: constant.endpoint.polyfill })
], PolyfillController.prototype, "onPolyfillRequested", null);

const config = Object.assign({}, environment, { development: environment.NODE_ENV == null || environment.NODE_ENV === "" || environment.NODE_ENV.toLowerCase() === "development", staging: environment.NODE_ENV != null && environment.NODE_ENV.toLowerCase() === "staging", production: environment.NODE_ENV != null && environment.NODE_ENV.toLowerCase() === "production", testing: booleanize(environment.TESTING), debug: booleanize(environment.DEBUG), verbose: booleanize(environment.VERBOSE), clearCache: booleanize(environment.CLEAR_CACHE), http2: booleanize(environment.HTTP2), https: booleanize(environment.HTTPS), host: environment.HOST, port: parseInt(environment.PORT), email: environment.EMAIL != null && environment.EMAIL !== "" ? environment.EMAIL : undefined, key: environment.KEY == null || environment.KEY === ""
        ? undefined
        : environment.KEY.trim().startsWith("-----BEGIN RSA PRIVATE KEY-----")
            ? buffer.Buffer.from(environment.KEY.replace(/\\n/g, "\n"))
            : fs.readFileSync(environment.KEY), cert: environment.CERT == null || environment.CERT === ""
        ? undefined
        : environment.CERT.trim().startsWith("-----BEGIN CERTIFICATE-----")
            ? buffer.Buffer.from(environment.CERT.replace(/\\n/g, "\n"))
            : fs.readFileSync(environment.CERT) });

const brotliCompressionOptions = {
    mode: 0,
    quality: 11,
    lgwin: 22
};

const zlibCompressionOptions = {
    level: 9
};

const container = new di.DIContainer();
// Utilities
container.registerSingleton(undefined, { identifier: "IFileLoader", implementation: fileloader.FileLoader });
container.registerSingleton(undefined, { identifier: "IFileSaver", implementation: filesaver.FileSaver });
// Services
container.registerSingleton(undefined, { identifier: "ILoggerService", implementation: LoggerService });
container.registerSingleton(undefined, { identifier: "ICompressorService", implementation: CompressorService });
container.registerSingleton(undefined, { identifier: "IMinifyService", implementation: MinifyService });
container.registerSingleton(undefined, { identifier: "IFlattenerService", implementation: FlattenerService });
container.registerSingleton(undefined, { identifier: "IMemoryRegistryService", implementation: MemoryRegistryService });
container.registerSingleton(undefined, { identifier: "ICacheRegistryService", implementation: CacheRegistryService });
container.registerSingleton(undefined, { identifier: "IPolyfillBuilderService", implementation: PolyfillBuilderService });
container.registerSingleton(undefined, { identifier: "IApiService", implementation: ApiService });
// Configuration
container.registerSingleton(() => config, { identifier: "IConfig" });
container.registerSingleton(() => brotliCompressionOptions, { identifier: "BrotliEncodeParams" });
container.registerSingleton(() => zlibCompressionOptions, { identifier: "ZlibOptions" });
// Server
container.registerSingleton(undefined, { identifier: "IServer", implementation: Server });
container.registerSingleton(undefined, { identifier: "IRequestHandler", implementation: RequestHandler });
// Business Logic
container.registerSingleton(undefined, { identifier: "IStaticBl", implementation: StaticBl });
container.registerSingleton(undefined, { identifier: "IPolyfillBl", implementation: PolyfillBl });
// Controller
container.registerSingleton(undefined, { identifier: "IStaticController", implementation: StaticController });
container.registerSingleton(undefined, { identifier: "IPolyfillController", implementation: PolyfillController });
container.registerSingleton(() => [container.get({ identifier: "IStaticController" }), container.get({ identifier: "IPolyfillController" })], { identifier: "RegisteredControllers" });

// Initializes the Cache Registry and then launches the server
container
    .get({ identifier: "ICacheRegistryService" })
    .initialize()
    .then(async () => container.get({ identifier: "IApiService" }).launch());

exports.polyfillRawForceName = polyfillRawForceName;
exports.polyfillOptionValueSeparator = polyfillOptionValueSeparator;
exports.polyfillRawDivider = polyfillRawDivider;
exports.polyfillRawSeparator = polyfillRawSeparator;
//# sourceMappingURL=index.js.map
